from django.http import HttpResponseRedirect
from django.urls import reverse
from django.utils.http import urlencode

from allauth.socialaccount.providers.base import Provider, ProviderAccount


class SAMLAccount(ProviderAccount):
    pass


class SAMLProvider(Provider):
    id = "saml"
    name = "SAML"
    supports_redirect = True
    account_class = SAMLAccount
    default_attribute_mapping = {
        "uid": [
            "urn:oasis:names:tc:SAML:attribute:subject-id",
        ],
        "email": [
            "urn:oid:0.9.2342.19200300.100.1.3",
            "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/emailaddress",
        ],
        "email_verified": [
            "http://schemas.auth0.com/email_verified",
        ],
        "first_name": [
            "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/givenname",
            "urn:oid:2.5.4.42",
        ],
        "last_name": [
            "urn:oid:2.5.4.4",
        ],
        "username": [
            "http://schemas.auth0.com/nickname",
        ],
    }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.name = self.app.name or self.app.client_id or self.name

    def get_login_url(self, request, **kwargs):
        url = reverse("saml_login", kwargs={"organization_slug": self.app.client_id})
        if kwargs:
            url = f"{url}?{urlencode(kwargs)}"
        return url

    def extract_extra_data(self, data):
        return data.get_attributes()

    def extract_uid(self, data):
        """https://docs.oasis-open.org/security/saml-subject-id-attr/v1.0/csprd01/saml-subject-id-attr-v1.0-csprd01.html

        Quotes:

        "While the Attributes defined in this profile have as a goal the
        explicit replacement of the <saml:NameID> element as a means of subject
        identification, it is certainly possible to compose them with existing
        NameID usage provided the same subject is being identified. This can
        also serve as a migration strategy for existing applications."


        "SAML does not define an identifier that meets all of these
        requirements well. It does standardize a kind of NameID termed
        “persistent” that meets some of them in the particular case of so-called
        “pairwise” identification, where an identifier varies by relying
        party. It has seen minimal adoption outside of a few contexts, and fails
        at the “compact” and “simple to handle” criteria above, on top of the
        disadvantages inherent with all NameID usage."

        Overall, our strategy is to prefer a uid resulting from explicit
        attribute mappings, and only if there is no such uid fallback to the
        NameID.
        """
        uid = self._extract(data).get("uid")
        if uid is None:
            uid = data.get_nameid()
        return uid

    def extract_common_fields(self, data):
        ret = self._extract(data)
        ret.pop("uid", None)
        return ret

    def _extract(self, data):
        provider_config = self.app.settings
        raw_attributes = data.get_attributes()
        attributes = {}
        attribute_mapping = provider_config.get(
            "attribute_mapping", self.default_attribute_mapping
        )
        # map configured provider attributes
        for key, provider_keys in attribute_mapping.items():
            if isinstance(provider_keys, str):
                provider_keys = [provider_keys]
            for provider_key in provider_keys:
                attribute_list = raw_attributes.get(provider_key, None)
                if attribute_list is not None and len(attribute_list) > 0:
                    attributes[key] = attribute_list[0]
                    break
        email_verified = attributes.get("email_verified")
        if email_verified:
            email_verified = email_verified.lower() in ["true", "1", "t", "y", "yes"]
            attributes["email_verified"] = email_verified

        # If we did not find an email, check if the NameID contains the email.
        if not attributes.get("email") and (
            data.get_nameid_format()
            == "urn:oasis:names:tc:SAML:1.1:nameid-format:emailAddress"
            # Alternatively, if `use_id_for_email` is true, then we always interpret the nameID as email
            or provider_config.get("use_nameid_for_email", False)
        ):
            attributes["email"] = data.get_nameid()

        return attributes

    def redirect(self, request, process, next_url=None, data=None, **kwargs):
        from allauth.socialaccount.providers.saml.utils import build_auth

        auth = build_auth(request, self)
        # If we pass `return_to=None` `auth.login` will use the URL of the
        # current view.
        redirect = auth.login(return_to="")
        self.stash_redirect_state(
            request,
            process,
            next_url,
            data,
            state_id=auth.get_last_request_id(),
            **kwargs,
        )
        return HttpResponseRedirect(redirect)


provider_classes = [SAMLProvider]



from allauth.socialaccount.providers.oauth2.urls import default_urlpatterns
from allauth.socialaccount.providers.soundcloud.provider import SoundCloudProvider


urlpatterns = default_urlpatterns(SoundCloudProvider)



from allauth import app_settings as allauth_settings
from allauth.account.models import EmailAddress


def wipe_password(request, user, email: str) -> None:
    """
    Consider a scenario where an attacker signs up for an account using the
    email address of a victim. Obviously, the email address cannot be
    verified, yet the attacker -- knowing the password -- can wait until the
    victim appears.  When the victim signs in using email authentication, it
    is not obvious that the victim is signing into an account that was not
    created by the victim. As a result, both the attacker and the victim now
    have access to the account. To prevent this, we wipe the password of the
    account in case the email address was not verified, effectively locking
    out the attacker.
    """
    try:
        address = EmailAddress.objects.get_for_user(user, email)
    except EmailAddress.DoesNotExist:
        address = None
    if address and address.verified:
        # Verified email address, no reason to worry.
        return
    if user.has_usable_password():
        user.set_unusable_password()
        user.save(update_fields=["password"])
    # Also wipe any other sessions (upstream integrators may hook up to the
    # ending of the sessions to trigger e.g. backchannel logout.
    if allauth_settings.USERSESSIONS_ENABLED:
        from allauth.usersessions.internal.flows.sessions import end_other_sessions

        end_other_sessions(request, user)



from django.urls import include, path

from allauth import app_settings as allauth_settings
from allauth.account import app_settings as account_settings
from allauth.headless.account import views


def build_urlpatterns(client):
    account_patterns = []
    auth_patterns = [
        path(
            "session",
            views.SessionView.as_api_view(client=client),
            name="current_session",
        ),
        path(
            "reauthenticate",
            views.ReauthenticateView.as_api_view(client=client),
            name="reauthenticate",
        ),
        path(
            "code/confirm",
            views.ConfirmLoginCodeView.as_api_view(client=client),
            name="confirm_login_code",
        ),
    ]
    if not allauth_settings.SOCIALACCOUNT_ONLY:
        account_patterns.extend(
            [
                path(
                    "password/change",
                    views.ChangePasswordView.as_api_view(client=client),
                    name="change_password",
                ),
                path(
                    "email",
                    views.ManageEmailView.as_api_view(client=client),
                    name="manage_email",
                ),
                path(
                    "phone",
                    views.ManagePhoneView.as_api_view(client=client),
                    name="manage_phone",
                ),
            ]
        )
        auth_patterns.extend(
            [
                path(
                    "password/",
                    include(
                        [
                            path(
                                "request",
                                views.RequestPasswordResetView.as_api_view(
                                    client=client
                                ),
                                name="request_password_reset",
                            ),
                            path(
                                "reset",
                                views.ResetPasswordView.as_api_view(client=client),
                                name="reset_password",
                            ),
                        ]
                    ),
                ),
                path(
                    "login",
                    views.LoginView.as_api_view(client=client),
                    name="login",
                ),
                path(
                    "signup",
                    views.SignupView.as_api_view(client=client),
                    name="signup",
                ),
                path(
                    "email/verify",
                    views.VerifyEmailView.as_api_view(client=client),
                    name="verify_email",
                ),
                path(
                    "email/verify/resend",
                    views.ResendEmailVerificationCodeView.as_api_view(client=client),
                    name="resend_email_verification_code",
                ),
                path(
                    "phone/verify",
                    views.VerifyPhoneView.as_api_view(client=client),
                    name="verify_phone",
                ),
                path(
                    "phone/verify/resend",
                    views.ResendPhoneVerificationCodeView.as_api_view(client=client),
                    name="resend_phone_verification_code",
                ),
            ]
        )
    if account_settings.LOGIN_BY_CODE_ENABLED:
        auth_patterns.extend(
            [
                path(
                    "code/request",
                    views.RequestLoginCodeView.as_api_view(client=client),
                    name="request_login_code",
                ),
                path(
                    "code/resend",
                    views.ResendLoginCodeView.as_api_view(client=client),
                    name="resend_login_code",
                ),
            ]
        )
    return [
        path("auth/", include(auth_patterns)),
        path(
            "account/",
            include(account_patterns),
        ),
    ]



from django.urls import include, path, re_path

from allauth.socialaccount import app_settings

from . import views


urlpatterns = [
    re_path(
        r"^(?P<provider_id>[^/]+)/",
        include(
            [
                path(
                    "login/",
                    views.login,
                    name="openid_connect_login",
                ),
                path(
                    "login/callback/",
                    views.callback,
                    name="openid_connect_callback",
                ),
            ]
        ),
    )
]

if app_settings.OPENID_CONNECT_URL_PREFIX:
    urlpatterns = [
        path(f"{app_settings.OPENID_CONNECT_URL_PREFIX}/", include(urlpatterns))
    ]



from django.urls import include, path

from allauth.headless.constants import Client
from allauth.headless.mfa import views
from allauth.mfa import app_settings as mfa_settings


def build_urlpatterns(client):
    auth_patterns = [
        path(
            "2fa/authenticate",
            views.AuthenticateView.as_api_view(client=client),
            name="authenticate",
        ),
        path(
            "2fa/reauthenticate",
            views.ReauthenticateView.as_api_view(client=client),
            name="reauthenticate",
        ),
    ]
    if mfa_settings.TRUST_ENABLED and client == Client.BROWSER:
        auth_patterns.append(
            path(
                "2fa/trust",
                views.TrustView.as_api_view(client=client),
                name="trust",
            )
        )

    authenticators = []
    if "totp" in mfa_settings.SUPPORTED_TYPES:
        authenticators.append(
            path(
                "totp",
                views.ManageTOTPView.as_api_view(client=client),
                name="manage_totp",
            )
        )
    if "recovery_codes" in mfa_settings.SUPPORTED_TYPES:
        authenticators.append(
            path(
                "recovery-codes",
                views.ManageRecoveryCodesView.as_api_view(client=client),
                name="manage_recovery_codes",
            )
        )
    if "webauthn" in mfa_settings.SUPPORTED_TYPES:
        authenticators.extend(
            [
                path(
                    "webauthn",
                    views.ManageWebAuthnView.as_api_view(client=client),
                    name="manage_webauthn",
                ),
            ]
        )
        auth_patterns.extend(
            [
                path(
                    "webauthn/authenticate",
                    views.AuthenticateWebAuthnView.as_api_view(client=client),
                    name="authenticate_webauthn",
                ),
                path(
                    "webauthn/reauthenticate",
                    views.ReauthenticateWebAuthnView.as_api_view(client=client),
                    name="reauthenticate_webauthn",
                ),
            ]
        )
        if mfa_settings.PASSKEY_LOGIN_ENABLED:
            auth_patterns.append(
                path(
                    "webauthn/login",
                    views.LoginWebAuthnView.as_api_view(client=client),
                    name="login_webauthn",
                )
            )
        if mfa_settings.PASSKEY_SIGNUP_ENABLED:
            auth_patterns.append(
                path(
                    "webauthn/signup",
                    views.SignupWebAuthnView.as_api_view(client=client),
                    name="signup_webauthn",
                )
            )

    return [
        path(
            "auth/",
            include(auth_patterns),
        ),
        path(
            "account/",
            include(
                [
                    path(
                        "authenticators",
                        views.AuthenticatorsView.as_api_view(client=client),
                        name="authenticators",
                    ),
                    path(
                        "authenticators/",
                        include(authenticators),
                    ),
                ]
            ),
        ),
    ]



from allauth.socialaccount.providers.base import ProviderAccount
from allauth.socialaccount.providers.fxa.constants import PROVIDER_ID
from allauth.socialaccount.providers.fxa.views import FirefoxAccountsOAuth2Adapter
from allauth.socialaccount.providers.oauth2.provider import OAuth2Provider


class FirefoxAccountsAccount(ProviderAccount):
    pass


class FirefoxAccountsProvider(OAuth2Provider):
    id = PROVIDER_ID
    name = "Firefox Accounts"
    account_class = FirefoxAccountsAccount
    oauth2_adapter_class = FirefoxAccountsOAuth2Adapter

    def get_default_scope(self):
        return ["profile"]

    def extract_uid(self, data):
        return str(data["uid"])

    def extract_common_fields(self, data):
        return dict(email=data.get("email"))


provider_classes = [FirefoxAccountsProvider]



from allauth.socialaccount.providers.base import ProviderAccount
from allauth.socialaccount.providers.meetup.views import MeetupOAuth2Adapter
from allauth.socialaccount.providers.oauth2.provider import OAuth2Provider


class MeetupAccount(ProviderAccount):
    pass


class MeetupProvider(OAuth2Provider):
    id = "meetup"
    name = "Meetup"
    account_class = MeetupAccount
    oauth2_adapter_class = MeetupOAuth2Adapter

    def extract_uid(self, data):
        return str(data["id"])

    def extract_common_fields(self, data):
        return dict(email=data.get("email"), name=data.get("name"))


provider_classes = [MeetupProvider]



from allauth.socialaccount import app_settings
from allauth.socialaccount.adapter import get_adapter
from allauth.socialaccount.providers.oauth2.views import (
    OAuth2Adapter,
    OAuth2CallbackView,
    OAuth2LoginView,
)


class EdxOAuth2Adapter(OAuth2Adapter):
    provider_id = "edx"
    provider_default_url = "https://edx.org"

    settings = app_settings.PROVIDERS.get(provider_id, {})
    provider_base_url = settings.get("EDX_URL", provider_default_url)

    access_token_url = f"{provider_base_url}/oauth2/access_token"
    authorize_url = f"{provider_base_url}/oauth2/authorize/"
    profile_url = f"{provider_base_url}/api/user/v1/me"
    account_url = "{0}/api/user/v1/accounts/{1}"
    supports_state = False
    redirect_uri_protocol = "https"

    def complete_login(self, request, app, token, **kwargs):
        headers = {"Authorization": f"Bearer {token.token}"}
        with get_adapter().get_requests_session() as sess:
            response = sess.get(self.profile_url, headers=headers)
            extra_data = response.json()

            if extra_data.get("email", None) is None:
                account_url = self.account_url.format(
                    self.provider_base_url, extra_data["username"]
                )
                response = sess.get(account_url, headers=headers)
                extra_data = response.json()

        return self.get_provider().sociallogin_from_response(request, extra_data)


oauth2_login = OAuth2LoginView.adapter_view(EdxOAuth2Adapter)
oauth2_callback = OAuth2CallbackView.adapter_view(EdxOAuth2Adapter)



from allauth.socialaccount.providers.oauth2.urls import default_urlpatterns
from allauth.socialaccount.providers.stackexchange.provider import StackExchangeProvider


urlpatterns = default_urlpatterns(StackExchangeProvider)



import uuid
from requests import RequestException

from allauth.socialaccount.adapter import get_adapter
from allauth.socialaccount.providers.oauth2.views import (
    OAuth2Adapter,
    OAuth2CallbackView,
    OAuth2LoginView,
)
from allauth.utils import get_request_param


class VKOAuth2Adapter(OAuth2Adapter):
    provider_id = "vk"
    access_token_url = "https://id.vk.ru/oauth2/auth"  # nosec
    authorize_url = "https://id.vk.ru/authorize"
    profile_url = "https://id.vk.ru/oauth2/user_info"

    def get_access_token_data(self, request, app, client, pkce_code_verifier=None):
        code = get_request_param(self.request, "code")
        device_id = get_request_param(self.request, "device_id")
        extra_data = {
            # "state" isn't strictly necessary for now, but since VK ID documentation doesn't
            # specify required vs optional parameters at all, we still add this (now optional)
            # param to make the code less fragile for future, when they may start requiring it
            "state": str(uuid.uuid4()),
            "device_id": device_id,
        }
        data = client.get_access_token(
            code, pkce_code_verifier=pkce_code_verifier, extra_data=extra_data
        )
        self.did_fetch_access_token = True
        return data

    def complete_login(self, request, app, token, **kwargs):
        req_data = {
            "access_token": token.token,
            "client_id": app.client_id,
        }
        with get_adapter().get_requests_session() as sess:
            resp = sess.post(self.profile_url, data=req_data)
            resp.raise_for_status()
            resp_data = resp.json()
        if "error" in resp_data or "user" not in resp_data:
            raise RequestException(
                "Could not get basic data for user being authenticated"
            )
        return self.get_provider().sociallogin_from_response(request, resp_data["user"])


oauth2_login = OAuth2LoginView.adapter_view(VKOAuth2Adapter)
oauth2_callback = OAuth2CallbackView.adapter_view(VKOAuth2Adapter)



from allauth.socialaccount.adapter import get_adapter
from allauth.socialaccount.providers.oauth2.views import (
    OAuth2Adapter,
    OAuth2CallbackView,
    OAuth2LoginView,
)


class StackExchangeOAuth2Adapter(OAuth2Adapter):
    provider_id = "stackexchange"
    access_token_url = "https://stackexchange.com/oauth/access_token"  # nosec
    authorize_url = "https://stackexchange.com/oauth"
    profile_url = "https://api.stackexchange.com/2.1/me"

    def complete_login(self, request, app, token, **kwargs):
        provider = self.get_provider()
        site = provider.get_site()
        with get_adapter().get_requests_session() as sess:
            params = {"access_token": token.token, "key": app.key, "site": site}
            resp = sess.get(self.profile_url, params=params)
            resp.raise_for_status()
            extra_data = resp.json()["items"][0]
        return self.get_provider().sociallogin_from_response(request, extra_data)


oauth2_login = OAuth2LoginView.adapter_view(StackExchangeOAuth2Adapter)
oauth2_callback = OAuth2CallbackView.adapter_view(StackExchangeOAuth2Adapter)



from rest_framework.permissions import BasePermission

from allauth.idp.oidc.internal.scope import is_scope_granted
from allauth.idp.oidc.models import Token


class TokenPermission(BasePermission):
    scope = None

    def has_permission(self, request, view) -> bool:
        access_token = request.auth
        if (
            not isinstance(access_token, Token)
            or access_token.type != Token.Type.ACCESS_TOKEN
        ):
            return False
        return is_scope_granted(self.scope, access_token, request.method)

    @classmethod
    def has_scope(cls, scope: str | list | dict):
        """
        Constructs and returns specific permission **class** (not instance)
        that checks that the request is authenticated by means of a token (see:
        ``TokenAuthentication``), and, that this token has the specified
        ``scope`` granted.

        The scope passed can either be:

        - a single scope (``str``),
        - a list of scopes, all of which should be granted.
        - a list of scope lists. Your token should match at least all scopes of one of the scope lists.
        - A dictionary, with the request method (e.g. ``GET``) as key, and one
          of the scope values from the previous bullet. The scopes to match are
          then dynamically selected based on the request.

        """

        class TokenHasScopePermission(cls):  # type: ignore[valid-type,misc]
            def __init__(self, *args, **kwargs):
                super().__init__(*args, **kwargs)
                self.scope = scope

        return TokenHasScopePermission



from allauth.socialaccount.providers.amazon.provider import AmazonProvider
from allauth.socialaccount.providers.oauth2.urls import default_urlpatterns


urlpatterns = default_urlpatterns(AmazonProvider)



import base64
import json

from django.core.exceptions import FieldDoesNotExist, ImproperlyConfigured
from django.core.files.base import ContentFile
from django.core.serializers.json import DjangoJSONEncoder
from django.db.models import FileField
from django.db.models.fields import BinaryField, DateField, DateTimeField, TimeField
from django.utils import dateparse
from django.utils.encoding import force_bytes, force_str


SERIALIZED_DB_FIELD_PREFIX = "_db_"


def serialize_instance(instance) -> dict:
    """
    Since Django 1.6 items added to the session are no longer pickled,
    but JSON encoded by default. We are storing partially complete models
    in the session (user, account, token, ...). We cannot use standard
    Django serialization, as these are models are not "complete" yet.
    Serialization will start complaining about missing relations et al.
    """
    data = {}
    for k, v in instance.__dict__.items():
        if k.startswith("_") or callable(v):
            continue
        try:
            field = instance._meta.get_field(k)
            if isinstance(field, BinaryField):
                if v is not None:
                    v = force_str(base64.b64encode(v))
            elif isinstance(field, FileField):
                if v and not isinstance(v, str):
                    v = {
                        "name": v.name,
                        "content": base64.b64encode(v.read()).decode("ascii"),
                    }
            # Check if the field is serializable. If not, we'll fall back
            # to serializing the DB values which should cover most use cases.
            try:
                json.dumps(v, cls=DjangoJSONEncoder)
            except TypeError:
                v = field.get_prep_value(v)
                k = SERIALIZED_DB_FIELD_PREFIX + k
        except FieldDoesNotExist:
            pass
        data[k] = v
    return json.loads(json.dumps(data, cls=DjangoJSONEncoder))


def deserialize_instance(model, data):
    if not isinstance(data, dict):
        raise ValueError()
    ret = model()
    for k, v in data.items():
        is_db_value = False
        if k.startswith(SERIALIZED_DB_FIELD_PREFIX):
            k = k[len(SERIALIZED_DB_FIELD_PREFIX) :]
            is_db_value = True
        if v is not None:
            try:
                f = model._meta.get_field(k)
                if isinstance(f, DateTimeField):
                    v = dateparse.parse_datetime(v)
                elif isinstance(f, TimeField):
                    v = dateparse.parse_time(v)
                elif isinstance(f, DateField):
                    v = dateparse.parse_date(v)
                elif isinstance(f, BinaryField):
                    v = force_bytes(base64.b64decode(force_bytes(v)))
                elif isinstance(f, FileField):
                    if isinstance(v, dict):
                        v = ContentFile(base64.b64decode(v["content"]), name=v["name"])
                elif is_db_value:
                    try:
                        # This is quite an ugly hack, but will cover most
                        # use cases...
                        # The signature of `from_db_value` changed in Django 3
                        # https://docs.djangoproject.com/en/3.0/releases/3.0/#features-removed-in-3-0
                        v = f.from_db_value(v, None, None)
                    except Exception:
                        raise ImproperlyConfigured(
                            f"Unable to auto serialize field '{k}', custom serialization override required"
                        )
            except FieldDoesNotExist:
                pass
        setattr(ret, k, v)
    return ret



from allauth.socialaccount.providers.microsoft.provider import MicrosoftGraphProvider
from allauth.socialaccount.providers.oauth2.urls import default_urlpatterns


urlpatterns = default_urlpatterns(MicrosoftGraphProvider)



from allauth.socialaccount.providers.mediawiki.provider import MediaWikiProvider
from allauth.socialaccount.providers.oauth2.urls import default_urlpatterns


urlpatterns = default_urlpatterns(MediaWikiProvider)



from allauth.socialaccount.providers.oauth.urls import default_urlpatterns
from allauth.socialaccount.providers.trello.provider import TrelloProvider


urlpatterns = default_urlpatterns(TrelloProvider)



from django.shortcuts import render

from allauth.account import app_settings as account_app_settings
from allauth.socialaccount import app_settings


def respond_to_login_on_get(request, provider):
    if (not app_settings.LOGIN_ON_GET) and request.method == "GET":
        return render(
            request,
            f"socialaccount/login.{account_app_settings.TEMPLATE_EXTENSION}",
            {
                "provider": provider,
                "process": request.GET.get("process"),
            },
        )



from allauth.socialaccount.providers.base import AuthAction, ProviderAccount
from allauth.socialaccount.providers.oauth2.provider import OAuth2Provider
from allauth.socialaccount.providers.ynab.views import YNABOAuth2Adapter


class Scope:
    ACCESS = "read-only"


class YNABAccount(ProviderAccount):
    pass


class YNABProvider(OAuth2Provider):
    id = "ynab"
    name = "YNAB"
    account_class = YNABAccount
    oauth2_adapter_class = YNABOAuth2Adapter

    def get_default_scope(self):
        scope = [Scope.ACCESS]
        return scope

    def get_auth_params_from_request(self, request, action):
        ret = super().get_auth_params_from_request(request, action)
        if action == AuthAction.REAUTHENTICATE:
            ret["prompt"] = "select_account consent"
        return ret

    def extract_uid(self, data):
        return str(data["data"]["user"]["id"])


provider_classes = [YNABProvider]



from allauth.socialaccount.adapter import get_adapter
from allauth.socialaccount.providers.oauth2.views import (
    OAuth2Adapter,
    OAuth2CallbackView,
    OAuth2LoginView,
)


class DisqusOAuth2Adapter(OAuth2Adapter):
    provider_id = "disqus"
    access_token_url = "https://disqus.com/api/oauth/2.0/access_token/"  # nosec
    authorize_url = "https://disqus.com/api/oauth/2.0/authorize/"
    profile_url = "https://disqus.com/api/3.0/users/details.json"
    scope_delimiter = ","

    def complete_login(self, request, app, token, **kwargs):
        with get_adapter().get_requests_session() as sess:
            params = {
                "access_token": token.token,
                "api_key": app.client_id,
                "api_secret": app.secret,
            }
            resp = sess.get(self.profile_url, params=params)
            resp.raise_for_status()
            extra_data = resp.json().get("response")

        login = self.get_provider().sociallogin_from_response(request, extra_data)
        return login


oauth2_login = OAuth2LoginView.adapter_view(DisqusOAuth2Adapter)
oauth2_callback = OAuth2CallbackView.adapter_view(DisqusOAuth2Adapter)



import unicodedata
from collections import OrderedDict

from django.contrib.auth import REDIRECT_FIELD_NAME, get_user_model
from django.db import models
from django.db.models import Q
from django.utils.http import base36_to_int, int_to_base36

from allauth.account import app_settings
from allauth.account.adapter import get_adapter
from allauth.account.internal import flows
from allauth.account.internal.emailkit import valid_email_or_none
from allauth.account.internal.userkit import default_user_display  # noqa
from allauth.account.internal.userkit import user_display  # noqa
from allauth.account.internal.userkit import user_email  # noqa
from allauth.account.internal.userkit import user_field  # noqa
from allauth.account.internal.userkit import user_username  # noqa
from allauth.account.models import Login
from allauth.core.internal import httpkit
from allauth.utils import get_request_param


def _unicode_ci_compare(s1: str, s2: str) -> bool:
    """
    Perform case-insensitive comparison of two identifiers, using the
    recommended algorithm from Unicode Technical Report 36, section
    2.11.2(B)(2).
    """
    norm_s1 = unicodedata.normalize("NFKC", s1).casefold()
    norm_s2 = unicodedata.normalize("NFKC", s2).casefold()
    return norm_s1 == norm_s2


def get_next_redirect_url(
    request, redirect_field_name=REDIRECT_FIELD_NAME
) -> str | None:
    """
    Returns the next URL to redirect to, if it was explicitly passed
    via the request.
    """
    redirect_to = get_request_param(request, redirect_field_name)
    if redirect_to and not get_adapter().is_safe_url(redirect_to):
        redirect_to = None
    return redirect_to


def get_login_redirect_url(
    request, url=None, redirect_field_name=REDIRECT_FIELD_NAME, signup=False
) -> str:
    ret = url
    if url and callable(url):
        # In order to be able to pass url getters around that depend
        # on e.g. the authenticated state.
        ret = url()
    if not ret:
        ret = get_next_redirect_url(request, redirect_field_name=redirect_field_name)
    if not ret:
        if signup:
            ret = get_adapter().get_signup_redirect_url(request)
        else:
            ret = get_adapter().get_login_redirect_url(request)
    return ret


def has_verified_email(user, email=None) -> bool:
    from .models import EmailAddress

    emailaddress = None
    if email:
        ret = False
        try:
            emailaddress = EmailAddress.objects.get_for_user(user, email)
            ret = emailaddress.verified
        except EmailAddress.DoesNotExist:
            pass
    else:
        ret = EmailAddress.objects.filter(user=user, verified=True).exists()
    return ret


def perform_login(
    request,
    user,
    email_verification=None,
    redirect_url=None,
    signal_kwargs=None,
    signup=False,
    email=None,
):
    login = Login(
        user=user,
        email_verification=email_verification,
        redirect_url=redirect_url,
        signal_kwargs=signal_kwargs,
        signup=signup,
        email=email,
    )
    return flows.login.perform_login(request, login)


def complete_signup(request, user, email_verification, success_url, signal_kwargs=None):
    return flows.signup.complete_signup(
        request,
        user=user,
        email_verification=email_verification,
        redirect_url=success_url,
        signal_kwargs=signal_kwargs,
    )


def cleanup_email_addresses(request, addresses):
    """
    Takes a list of EmailAddress instances and cleans it up, making
    sure only valid ones remain, without multiple primaries etc.

    Order is important: e.g. if multiple primary email addresses
    exist, the first one encountered will be kept as primary.
    """
    from .models import EmailAddress

    adapter = get_adapter()
    # Let's group by `email`
    e2a = OrderedDict()  # maps email to EmailAddress
    primary_addresses = []
    verified_addresses = []
    primary_verified_addresses = []
    for address in addresses:
        # Pick up only valid ones...
        email = valid_email_or_none(address.email)
        if not email:
            continue
        address.email = email  # `valid_email_or_none` lower cases
        # ... and non-conflicting ones...
        if (
            app_settings.UNIQUE_EMAIL
            and app_settings.PREVENT_ENUMERATION != "strict"
            and EmailAddress.objects.lookup([email])
        ):
            # Email address already exists.
            continue
        if (
            app_settings.UNIQUE_EMAIL
            and app_settings.PREVENT_ENUMERATION == "strict"
            and address.verified
            and EmailAddress.objects.is_verified(email)
        ):
            # Email address already exists, and is verified as well.
            continue
        a = e2a.get(email)
        if a:
            a.primary = a.primary or address.primary
            a.verified = a.verified or address.verified
        else:
            a = address
            a.verified = a.verified or adapter.is_email_verified(request, a.email)
            e2a[email] = a
        if a.primary:
            primary_addresses.append(a)
            if a.verified:
                primary_verified_addresses.append(a)
        if a.verified:
            verified_addresses.append(a)
    # Now that we got things sorted out, let's assign a primary
    if primary_verified_addresses:
        primary_address = primary_verified_addresses[0]
    elif verified_addresses:
        # Pick any verified as primary
        primary_address = verified_addresses[0]
    elif primary_addresses:
        # Okay, let's pick primary then, even if unverified
        primary_address = primary_addresses[0]
    elif e2a:
        # Pick the first
        primary_address = list(e2a.values())[0]
    else:
        # Empty
        primary_address = None
    # There can only be one primary
    for a in e2a.values():
        a.primary = primary_address.email.lower() == a.email.lower()
    return list(e2a.values()), primary_address


def setup_user_email(request, user, addresses):
    """
    Creates proper EmailAddress for the user that was just signed
    up. Only sets up, doesn't do any other handling such as sending
    out email confirmation mails etc.
    """
    from .models import EmailAddress

    assert not EmailAddress.objects.filter(user=user).exists()  # nosec
    priority_addresses = []
    # Is there a stashed email?
    adapter = get_adapter()
    stashed_email = adapter.unstash_verified_email(request)
    if stashed_email:
        priority_addresses.append(
            EmailAddress(
                user=user, email=stashed_email.lower(), primary=True, verified=True
            )
        )
    email = user_email(user)
    if email:
        priority_addresses.append(
            EmailAddress(user=user, email=email.lower(), primary=True, verified=False)
        )
    addresses, primary = cleanup_email_addresses(
        request, priority_addresses + addresses
    )
    for a in addresses:
        a.user = user
        a.save()
    EmailAddress.objects.fill_cache_for_user(user, addresses)
    if primary and (email or "").lower() != primary.email.lower():
        user_email(user, primary.email)
        user.save()
    return primary


def filter_users_by_username(*username):
    if app_settings.PRESERVE_USERNAME_CASING:
        qlist = [
            Q(**{f"{app_settings.USER_MODEL_USERNAME_FIELD}__iexact": u})
            for u in username
        ]
        q = qlist[0]
        for q2 in qlist[1:]:
            q = q | q2
        ret = get_user_model()._default_manager.filter(q)
    else:
        ret = get_user_model()._default_manager.filter(
            **{
                f"{app_settings.USER_MODEL_USERNAME_FIELD}__in": [
                    u.lower() for u in username
                ]
            }
        )
    return ret


def filter_users_by_email(
    email: str, is_active: bool | None = None, prefer_verified: bool = False
) -> list:
    """Return list of users by email address

    Typically one, at most just a few in length.  First we look through
    EmailAddress table, than customisable User model table. Add results
    together avoiding SQL joins and deduplicate.

    `prefer_verified`: When looking up users by email, there can be cases where
    users with verified email addresses are preferable above users who did not
    verify their email address. The password reset is such a use case -- if
    there is a user with a verified email than that user should be returned, not
    one of the other users.
    """
    from .models import EmailAddress

    User = get_user_model()
    email = email.lower()
    mails = list(EmailAddress.objects.filter(email=email).select_related("user"))
    is_verified = False
    if prefer_verified:
        verified_mails = list(filter(lambda e: e.verified, mails))
        if verified_mails:
            mails = verified_mails
            is_verified = True
    users = []
    for e in mails:
        if _unicode_ci_compare(e.email, email):
            users.append(e.user)
    if app_settings.USER_MODEL_EMAIL_FIELD and not is_verified:
        q_dict = {app_settings.USER_MODEL_EMAIL_FIELD: email}
        user_qs = User.objects.filter(**q_dict)
        for user in user_qs.iterator(2000):
            user_email = getattr(user, app_settings.USER_MODEL_EMAIL_FIELD)
            if _unicode_ci_compare(user_email, email):
                users.append(user)
    if is_active is not None:
        users = [u for u in set(users) if u.is_active == is_active]
    return list(set(users))


def passthrough_next_redirect_url(request, url: str, redirect_field_name: str) -> str:
    next_url = get_next_redirect_url(request, redirect_field_name)
    if next_url:
        url = httpkit.add_query_params(url, {redirect_field_name: next_url})
    return url


def user_pk_to_url_str(user) -> str:
    """
    This should return a string.
    """
    User = get_user_model()
    pk_field_class = type(User._meta.pk)
    if issubclass(pk_field_class, models.UUIDField):
        if isinstance(user.pk, str):
            return user.pk
        return user.pk.hex
    elif issubclass(pk_field_class, models.IntegerField):
        return int_to_base36(int(user.pk))
    return str(user.pk)


def url_str_to_user_pk(pk_str):
    User = get_user_model()
    remote_field = getattr(User._meta.pk, "remote_field", None)
    if remote_field and getattr(remote_field, "to", None):
        pk_field = User._meta.pk.remote_field.to._meta.pk
    else:
        pk_field = User._meta.pk
    pk_field_class = type(pk_field)
    if issubclass(pk_field_class, models.IntegerField):
        pk = base36_to_int(pk_str)
        # always call to_python() -- because there are fields like HashidField
        # that derive from IntegerField.
        pk = pk_field.to_python(pk)
    else:
        pk = pk_field.to_python(pk_str)
    return pk



import time

from django.http import HttpResponseRedirect
from django.urls import reverse

from allauth.account import app_settings
from allauth.account.models import Login
from allauth.account.stages import LoginStage, LoginStageController


LOGIN_SESSION_KEY = "account_login"


def get_pending_stage(request) -> LoginStage | None:
    stage = None
    if not request.user.is_authenticated:
        login = unstash_login(request, peek=True)
        if login:
            ctrl = LoginStageController(request, login)
            stage = ctrl.get_pending_stage()
    return stage


def redirect_to_pending_stage(request, stage: LoginStage):
    if stage.urlname:
        return HttpResponseRedirect(reverse(stage.urlname))
    clear_login(request)
    return HttpResponseRedirect(reverse("account_login"))


def clear_login(request) -> None:
    request.session.pop(LOGIN_SESSION_KEY, None)


def unstash_login(request, peek: bool = False):
    login = None
    if peek:
        data = request.session.get(LOGIN_SESSION_KEY)
    else:
        data = request.session.pop(LOGIN_SESSION_KEY, None)
    if isinstance(data, dict):
        try:
            login = Login.deserialize(data)
        except ValueError:
            pass
        else:
            if time.time() - login.initiated_at > app_settings.LOGIN_TIMEOUT:
                login = None
                clear_login(request)
            else:
                request._account_login_accessed = True
    return login


def stash_login(request, login) -> None:
    request.session[LOGIN_SESSION_KEY] = login.serialize()
    request._account_login_accessed = True



from urllib.parse import urlparse

from django.core.cache import cache
from django.core.exceptions import ImproperlyConfigured
from django.http import Http404
from django.urls import reverse
from django.utils.http import urlencode

from onelogin.saml2.auth import OneLogin_Saml2_Auth
from onelogin.saml2.constants import OneLogin_Saml2_Constants
from onelogin.saml2.idp_metadata_parser import OneLogin_Saml2_IdPMetadataParser

from allauth.account.adapter import get_adapter as get_account_adapter
from allauth.socialaccount.adapter import get_adapter
from allauth.socialaccount.models import SocialApp
from allauth.socialaccount.providers.saml.provider import SAMLProvider


def get_app_or_404(request, organization_slug):
    adapter = get_adapter()
    try:
        return adapter.get_app(
            request, provider=SAMLProvider.id, client_id=organization_slug
        )
    except SocialApp.DoesNotExist:
        raise Http404(f"no SocialApp found with client_id={organization_slug}")


def prepare_django_request(request) -> dict:
    result = {
        "https": "on" if request.is_secure() else "off",
        "http_host": request.META["HTTP_HOST"],
        "script_name": request.META.get("SCRIPT_NAME"),
        "path_info": request.META["PATH_INFO"],
        "get_data": request.GET.copy(),
        # 'lowercase_urlencoding': True,
        "post_data": request.POST.copy(),
    }
    return result


def build_sp_config(request, provider_config, org) -> dict:
    acs_url = request.build_absolute_uri(reverse("saml_acs", args=[org]))
    sls_url = request.build_absolute_uri(reverse("saml_sls", args=[org]))
    metadata_url = request.build_absolute_uri(reverse("saml_metadata", args=[org]))
    # SP entity ID generated with the following precedence:
    # 1. Explicitly configured SP via the SocialApp.settings
    # 2. Fallback to the SAML metadata urlpattern
    _sp_config = provider_config.get("sp", {})
    sp_entity_id = _sp_config.get("entity_id")
    sp_config = {
        "entityId": sp_entity_id or metadata_url,
        "assertionConsumerService": {
            "url": acs_url,
            "binding": OneLogin_Saml2_Constants.BINDING_HTTP_POST,
        },
        "singleLogoutService": {
            "url": sls_url,
            "binding": OneLogin_Saml2_Constants.BINDING_HTTP_REDIRECT,
        },
    }
    avd = provider_config.get("advanced", {})
    if avd.get("x509cert") is not None:
        sp_config["x509cert"] = avd["x509cert"]

    if avd.get("x509cert_new"):
        sp_config["x509certNew"] = avd["x509cert_new"]

    if avd.get("private_key") is not None:
        sp_config["privateKey"] = avd["private_key"]

    if avd.get("name_id_format") is not None:
        sp_config["NameIDFormat"] = avd["name_id_format"]

    return sp_config


def fetch_metadata_url_config(idp_config):
    metadata_url = idp_config["metadata_url"]
    entity_id = idp_config["entity_id"]
    cache_key = f"saml.metadata.{metadata_url}.{entity_id}"
    saml_config = cache.get(cache_key)
    if saml_config is None:
        saml_config = OneLogin_Saml2_IdPMetadataParser.parse_remote(
            metadata_url,
            entity_id=entity_id,
            timeout=idp_config.get("metadata_request_timeout", 10),
        )
        cache.set(
            cache_key,
            saml_config,
            idp_config.get("metadata_cache_timeout", 60 * 60 * 4),
        )
    return saml_config


def build_saml_config(request, provider_config, org) -> dict:
    avd = provider_config.get("advanced", {})
    security_config = {
        "authnRequestsSigned": avd.get("authn_request_signed", False),
        "digestAlgorithm": avd.get("digest_algorithm", OneLogin_Saml2_Constants.SHA256),
        "logoutRequestSigned": avd.get("logout_request_signed", False),
        "logoutResponseSigned": avd.get("logout_response_signed", False),
        "requestedAuthnContext": False,
        "signatureAlgorithm": avd.get(
            "signature_algorithm", OneLogin_Saml2_Constants.RSA_SHA256
        ),
        "signMetadata": avd.get("metadata_signed", False),
        "wantAssertionsEncrypted": avd.get("want_assertion_encrypted", False),
        "wantAssertionsSigned": avd.get("want_assertion_signed", False),
        "wantMessagesSigned": avd.get("want_message_signed", False),
        "nameIdEncrypted": avd.get("name_id_encrypted", False),
        "wantNameIdEncrypted": avd.get("want_name_id_encrypted", False),
        "allowSingleLabelDomains": avd.get("allow_single_label_domains", False),
        "rejectDeprecatedAlgorithm": avd.get("reject_deprecated_algorithm", True),
        "wantNameId": avd.get("want_name_id", False),
        "wantAttributeStatement": avd.get("want_attribute_statement", True),
        "allowRepeatAttributeName": avd.get("allow_repeat_attribute_name", True),
        "metadataValidUntil": avd.get("metadata_valid_until", None),
        "metadataCacheDuration": avd.get("metadata_cache_duration", None),
    }
    saml_config = {
        "strict": avd.get("strict", True),
        "security": security_config,
    }

    contact_person = provider_config.get("contact_person")
    if contact_person:
        saml_config["contactPerson"] = contact_person

    organization = provider_config.get("organization")
    if organization:
        saml_config["organization"] = organization

    idp = provider_config.get("idp")
    if idp is None:
        raise ImproperlyConfigured("`idp` missing")
    metadata_url = idp.get("metadata_url")
    if metadata_url:
        meta_config = fetch_metadata_url_config(idp)
        saml_config["idp"] = meta_config["idp"]
    else:
        saml_config["idp"] = {
            "entityId": idp["entity_id"],
            "x509cert": idp["x509cert"],
            "singleSignOnService": {"url": idp["sso_url"]},
        }
        slo_url = idp.get("slo_url")
        if slo_url:
            saml_config["idp"]["singleLogoutService"] = {"url": slo_url}

    saml_config["sp"] = build_sp_config(request, provider_config, org)
    return saml_config


def encode_relay_state(state) -> str:
    params = {"state": state}
    return urlencode(params)


def decode_relay_state(relay_state):
    """According to the spec, RelayState need not be a URL, yet,
    ``onelogin.saml2` exposes it as ``return_to -- The target URL the user
    should be redirected to after login``. Also, for an IdP initiated login
    sometimes a URL is used.
    """
    next_url = None
    if relay_state:
        parts = urlparse(relay_state)
        if parts.scheme or parts.netloc or (parts.path and parts.path.startswith("/")):
            if get_account_adapter().is_safe_url(relay_state):
                next_url = relay_state
    return next_url


def build_auth(request, provider):
    req = prepare_django_request(request)
    config = build_saml_config(request, provider.app.settings, provider.app.client_id)
    auth = OneLogin_Saml2_Auth(req, config)
    return auth



from django.utils.http import urlencode

from allauth.socialaccount.adapter import get_adapter
from allauth.socialaccount.providers.oauth.views import (
    OAuthAdapter,
    OAuthCallbackView,
    OAuthLoginView,
)


class TrelloOAuthAdapter(OAuthAdapter):
    provider_id = "trello"
    request_token_url = "https://trello.com/1/OAuthGetRequestToken"  # nosec
    authorize_url = "https://trello.com/1/OAuthAuthorizeToken"
    access_token_url = "https://trello.com/1/OAuthGetAccessToken"  # nosec

    def complete_login(self, request, app, token, response):
        # we need to get the member id and the other information
        info_url = "{base}?{query}".format(
            base="https://api.trello.com/1/members/me",
            query=urlencode({"key": app.key, "token": response.get("oauth_token")}),
        )
        with get_adapter().get_requests_session() as sess:
            resp = sess.get(info_url)
            resp.raise_for_status()
            extra_data = resp.json()
        return self.get_provider().sociallogin_from_response(request, extra_data)


oauth_login = OAuthLoginView.adapter_view(TrelloOAuthAdapter)
oauth_callback = OAuthCallbackView.adapter_view(TrelloOAuthAdapter)



from django.conf import settings
from django.http import HttpResponse

from allauth import app_settings
from allauth.core.exceptions import RateLimited  # noqa
from allauth.core.internal import ratelimit as _impl
from allauth.core.internal.ratelimit import Rate  # noqa
from allauth.utils import import_callable


def clear(request, *, action: str, key=None, user=None) -> None:
    from allauth.account import app_settings

    _impl.clear(
        request=request,
        config=app_settings.RATE_LIMITS,
        action=action,
        key=key,
        user=user,
    )


def consume(
    request,
    *,
    action,
    key=None,
    user=None,
    dry_run: bool = False,
    raise_exception: bool = False,
) -> bool:
    """
    TODO: We ened to deprecate this module and keep rate limiting internal
    to allauth. This method is using `allauth.account.app_settings` as a
    hard-coded source of settings, which is bad for reusability elsewhere.
    """
    from allauth.account import app_settings

    usage = _impl.consume(
        request=request,
        config=app_settings.RATE_LIMITS,
        action=action,
        key=key,
        user=user,
        dry_run=dry_run,
        raise_exception=raise_exception,
    )
    if not usage:
        return False
    return True


def respond_429(request) -> HttpResponse:
    if app_settings.HEADLESS_ENABLED and hasattr(request.allauth, "headless"):
        from allauth.headless.base.response import RateLimitResponse

        return RateLimitResponse(request)

    try:
        handler429 = import_callable(f"{settings.ROOT_URLCONF}.handler429")
        handler429 = import_callable(handler429)
    except (ImportError, AttributeError):
        handler429 = _impl.handler429
    return handler429(request)


def consume_or_429(request, *args, **kwargs) -> HttpResponse | None:
    if not consume(request, *args, **kwargs):
        return respond_429(request)
    return None



from http import HTTPStatus
from typing import Any

from django.forms.utils import ErrorList
from django.http import JsonResponse
from django.utils.cache import add_never_cache_headers

from allauth.headless.internal import authkit, sessionkit


class APIResponse(JsonResponse):
    def __init__(
        self,
        request,
        errors=None,
        data=None,
        meta: dict | None = None,
        status: int = HTTPStatus.OK,
    ):
        d: dict[str, Any] = {"status": status}
        if data is not None:
            d["data"] = data
        meta = self._add_session_meta(request, meta)
        if meta is not None:
            d["meta"] = meta
        if errors:
            d["errors"] = errors
        super().__init__(d, status=status)
        add_never_cache_headers(self)

    def _add_session_meta(self, request, meta: dict | None) -> dict | None:
        session_token = sessionkit.expose_session_token(request)
        access_token_payload = authkit.expose_access_token(request)
        if session_token:
            meta = meta or {}
            meta["session_token"] = session_token
        if access_token_payload:
            meta = meta or {}
            meta.update(access_token_payload)
        return meta


class ErrorResponse(APIResponse):
    def __init__(
        self, request, exception=None, input=None, status=HTTPStatus.BAD_REQUEST
    ):
        errors = []
        if exception is not None:
            error_datas = ErrorList(exception.error_list).get_json_data()
            errors.extend(error_datas)
        if input is not None:
            for field, error_list in input.errors.items():
                error_datas = error_list.get_json_data()
                for error_data in error_datas:
                    if field != "__all__":
                        error_data["param"] = field
                errors.extend(error_datas)
        super().__init__(request, status=status, errors=errors)



from allauth.socialaccount import app_settings
from allauth.socialaccount.adapter import get_adapter
from allauth.socialaccount.providers.oauth2.views import (
    OAuth2Adapter,
    OAuth2CallbackView,
    OAuth2LoginView,
)


class ShareFileOAuth2Adapter(OAuth2Adapter):
    provider_id = "sharefile"
    settings = app_settings.PROVIDERS.get(provider_id, {})
    subdomain = settings.get("SUBDOMAIN", "secure")
    apicp = settings.get("APICP", "sharefile.com")

    provider_default_url = settings.get("DEFAULT_URL", "https://secure.sharefile.com")
    provider_default_api_url = f"https://{subdomain}.sf-api.com"
    provider_api_version = "v3"

    access_token_url = f"https://{subdomain}.{apicp}/oauth/token"
    refresh_token_url = f"https://{subdomain}.{apicp}/oauth/token"
    authorize_url = f"{provider_default_url}/oauth/authorize"
    profile_url = f"{provider_default_api_url}/sf/{provider_api_version}/Users"

    def complete_login(self, request, app, token, response):
        headers = {"Authorization": f"Bearer {token.token}"}
        with get_adapter().get_requests_session() as sess:
            resp = sess.get(self.profile_url, headers=headers)
            extra_data = resp.json()
        return self.get_provider().sociallogin_from_response(request, extra_data)


oauth2_login = OAuth2LoginView.adapter_view(ShareFileOAuth2Adapter)
oauth2_callback = OAuth2CallbackView.adapter_view(ShareFileOAuth2Adapter)



from allauth.socialaccount.providers.oauth2.urls import default_urlpatterns
from allauth.socialaccount.providers.shopify.provider import ShopifyProvider


urlpatterns = default_urlpatterns(ShopifyProvider)



# Generated by Django 3.2.20 on 2023-10-11 09:23

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("socialaccount", "0005_socialtoken_nullable_app"),
    ]

    operations = [
        migrations.AlterField(
            model_name="socialaccount",
            name="extra_data",
            field=models.JSONField(default=dict, verbose_name="extra data"),
        ),
    ]



from django.urls import path

from allauth.socialaccount.providers.facebook.provider import FacebookProvider
from allauth.socialaccount.providers.oauth2.urls import default_urlpatterns

from . import views


urlpatterns = default_urlpatterns(FacebookProvider)

urlpatterns += [
    path(
        "facebook/login/token/",
        views.login_by_token,
        name="facebook_login_by_token",
    ),
]



from django.urls import path

from allauth.headless.base import views


def build_urlpatterns(client):
    return [
        path(
            "config",
            views.ConfigView.as_api_view(client=client),
            name="config",
        ),
    ]



from django.urls import include, path

from allauth import app_settings as allauth_settings
from allauth.headless import app_settings
from allauth.headless.account import urls as account_urls
from allauth.headless.base import urls as base_urls
from allauth.headless.constants import Client
from allauth.headless.tokens import urls as tokens_urls


def build_urlpatterns(client):
    patterns = []
    patterns.extend(base_urls.build_urlpatterns(client))
    patterns.append(
        path(
            "",
            include(
                (account_urls.build_urlpatterns(client), "headless"),
                namespace="account",
            ),
        )
    )
    if allauth_settings.SOCIALACCOUNT_ENABLED:
        from allauth.headless.socialaccount import urls as socialaccount_urls

        patterns.append(
            path(
                "",
                include(
                    (socialaccount_urls.build_urlpatterns(client), "headless"),
                    namespace="socialaccount",
                ),
            )
        )

    if allauth_settings.MFA_ENABLED:
        from allauth.headless.mfa import urls as mfa_urls

        patterns.append(
            path(
                "",
                include(
                    (mfa_urls.build_urlpatterns(client), "headless"),
                    namespace="mfa",
                ),
            )
        )

    if allauth_settings.USERSESSIONS_ENABLED:
        from allauth.headless.usersessions import urls as usersessions_urls

        patterns.append(
            path(
                "",
                include(
                    (usersessions_urls.build_urlpatterns(client), "headless"),
                    namespace="usersessions",
                ),
            )
        )

    if client == Client.APP:
        patterns.append(
            path(
                "",
                include(
                    (tokens_urls.build_urlpatterns(client), "headless"),
                    namespace="tokens",
                ),
            )
        )

    return [path("v1/", include(patterns))]


app_name = "headless"
urlpatterns = []
if Client.BROWSER in app_settings.CLIENTS:
    urlpatterns.append(
        path(
            "browser/",
            include(
                (build_urlpatterns(Client.BROWSER), "headless"),
                namespace="browser",
            ),
        )
    )
if Client.APP in app_settings.CLIENTS:
    urlpatterns.append(
        path(
            "app/",
            include((build_urlpatterns(Client.APP), "headless"), namespace="app"),
        )
    )

if app_settings.SERVE_SPECIFICATION:
    urlpatterns.append(
        path(
            "",
            include(
                "allauth.headless.spec.urls",
            ),
        )
    )



from pathlib import Path

from django.urls import reverse
from django.urls.exceptions import Resolver404

from allauth.account import app_settings as account_settings
from allauth.account.internal.flows.signup import base_signup_form_class
from allauth.core.internal.urlkit import script_aware_resolve
from allauth.headless import app_settings
from allauth.headless.adapter import get_adapter
from allauth.headless.spec.internal.openapikit import spec_for_dataclass, spec_for_field


def get_schema() -> dict:
    import yaml

    path = Path(__file__).parent.parent / "doc/openapi.yaml"
    with open(path, "rb") as f:
        spec = yaml.safe_load(f)

    path = Path(__file__).parent.parent / "doc/description.md"
    with open(path, "rb") as f:
        description = f.read().decode("utf8")

    spec["info"]["description"] = description

    specify_user(spec)
    chroot(spec)
    pin_client(spec)
    drop_unused_client_parameter(spec)
    used_tags = drop_unused_paths(spec)
    drop_unused_tags(spec, used_tags)
    drop_unused_tag_groups(spec, used_tags)
    specify_signup_fields(spec)
    specify_custom_signup_form(spec)
    return spec


def chroot(spec: dict) -> None:
    url = reverse("headless:openapi_yaml")
    root = url.rpartition("/")[0]
    paths = spec["paths"].items()
    spec["paths"] = {}
    for path, path_spec in paths:
        new_path = path.replace("/_allauth", root)
        spec["paths"][new_path] = path_spec


def pin_client(spec: dict) -> None:
    if len(app_settings.CLIENTS) != 1:
        return

    processed_paths = {}
    client_value = app_settings.CLIENTS[0]
    http_methods = [
        "get",
        "post",
        "put",
        "delete",
        "options",
        "head",
        "patch",
        "trace",
    ]

    def remove_client_param(parameters: list) -> list | None:
        filtered = [
            p
            for p in parameters
            if not (
                isinstance(p, dict)
                and p.get("$ref") == "#/components/parameters/Client"
            )
        ]
        return filtered or None

    for path_key, path_item in spec["paths"].items():
        current_path_item = dict(path_item)
        processed_path_key = path_key.replace("{client}", client_value)

        if "parameters" in current_path_item:
            new_params = remove_client_param(current_path_item["parameters"])
            if new_params:
                current_path_item["parameters"] = new_params
            else:
                current_path_item.pop("parameters")

        for method_name in http_methods:
            if method_name in current_path_item:
                operation_item = current_path_item[method_name]
                if isinstance(operation_item, dict) and "parameters" in operation_item:
                    new_params = remove_client_param(operation_item["parameters"])
                    if new_params:
                        operation_item["parameters"] = new_params
                    else:
                        operation_item.pop("parameters")

        processed_paths[processed_path_key] = current_path_item

    spec["paths"] = processed_paths


def drop_unused_client_parameter(spec: dict) -> None:
    if len(app_settings.CLIENTS) != 1:
        return

    if components := spec.get("components"):
        if parameters := components.get("parameters"):
            parameters.pop("Client", None)


def drop_unused_paths(spec: dict) -> set:
    paths = spec["paths"]
    used_tags = set()
    for path, path_spec in list(paths.items()):
        found_path = False
        for client in app_settings.CLIENTS:
            try:
                script_aware_resolve(path.replace("{client}", client))
                found_path = True
                break
            except Resolver404:
                pass
        if found_path:
            for method, method_spec in path_spec.items():
                used_tags.update(method_spec["tags"])
        else:
            paths.pop(path)
    return used_tags


def drop_unused_tags(spec: dict, used_tags: set) -> None:
    tags = spec["tags"]
    spec["tags"] = []
    for tag in tags:
        if tag["name"] not in used_tags:
            continue
        spec["tags"].append(tag)


def drop_unused_tag_groups(spec: dict, used_tags: set) -> None:
    tag_groups = spec["x-tagGroups"]
    spec["x-tagGroups"] = []
    for tag_group in tag_groups:
        if any([t in used_tags for t in tag_group["tags"]]):
            spec["x-tagGroups"].append(tag_group)


def specify_signup_fields(spec: dict) -> None:
    base_signup = spec["components"]["schemas"]["BaseSignup"]
    signup = spec["components"]["schemas"]["Signup"]
    properties = base_signup["properties"]
    required_fields = []
    for field_name in ("email", "phone", "username"):
        field = account_settings.SIGNUP_FIELDS.get(field_name)
        if not field:
            properties.pop(field_name)
        elif field["required"]:
            required_fields.append(field_name)
    base_signup["required"] = required_fields
    password_field = account_settings.SIGNUP_FIELDS.get("password1")
    if not password_field:
        signup["allOf"] = signup["allOf"][:1]
    elif not password_field["required"]:
        signup["allOf"][1]["required"].remove("password")


def specify_custom_signup_form(spec: dict) -> None:
    form_class = base_signup_form_class()
    base_signup = spec["components"]["schemas"]["BaseSignup"]
    for field_name, field in form_class.base_fields.items():
        is_required = hasattr(field, "required") and field.required
        field_spec = spec_for_field(field)
        if is_required:
            base_signup["required"].append(field_name)
        base_signup["properties"][field_name] = field_spec


def specify_user(spec: dict) -> None:
    dc = get_adapter().get_user_dataclass()
    schema, example = spec_for_dataclass(dc)
    spec["components"]["schemas"]["User"] = schema
    example_value = spec["components"]["examples"]["User"]["value"]
    example_value.clear()
    example_value.update(example)



from allauth.socialaccount.providers.base import ProviderAccount
from allauth.socialaccount.providers.oauth2.provider import OAuth2Provider
from allauth.socialaccount.providers.stripe.views import StripeOAuth2Adapter


class StripeAccount(ProviderAccount):
    def to_str(self):
        default = super().to_str()
        email = self.account.extra_data.get("email")
        business = self.account.extra_data.get("business_name")
        if business:
            return f"{email or default} ({business})"
        else:
            return email or default


class StripeProvider(OAuth2Provider):
    id = "stripe"
    name = "Stripe"
    account_class = StripeAccount
    oauth2_adapter_class = StripeOAuth2Adapter

    def extract_uid(self, data):
        return data["id"]

    def extract_common_fields(self, data):
        return dict(name=data.get("display_name"), email=data.get("email"))

    def get_default_scope(self):
        return ["read_only"]


provider_classes = [StripeProvider]



from allauth.socialaccount.adapter import get_adapter
from allauth.socialaccount.providers.oauth2.views import (
    OAuth2Adapter,
    OAuth2CallbackView,
    OAuth2LoginView,
)


class PaypalOAuth2Adapter(OAuth2Adapter):
    provider_id = "paypal"

    @property
    def authorize_url(self):
        path = "webapps/auth/protocol/openidconnect/v1/authorize"
        return f"https://www.{self._get_endpoint()}/{path}"

    @property
    def access_token_url(self):
        path = "v1/identity/openidconnect/tokenservice"
        return f"https://api.{self._get_endpoint()}/{path}"

    @property
    def profile_url(self):
        path = "v1/identity/openidconnect/userinfo"
        return f"https://api.{self._get_endpoint()}/{path}"

    def _get_endpoint(self):
        settings = self.get_provider().get_settings()
        if settings.get("MODE") == "live":
            return "paypal.com"
        else:
            return "sandbox.paypal.com"

    def complete_login(self, request, app, token, **kwargs):
        with get_adapter().get_requests_session() as sess:
            response = sess.post(
                self.profile_url,
                params={"schema": "openid", "access_token": token.token},
            )
            extra_data = response.json()
        return self.get_provider().sociallogin_from_response(request, extra_data)


oauth2_login = OAuth2LoginView.adapter_view(PaypalOAuth2Adapter)
oauth2_callback = OAuth2CallbackView.adapter_view(PaypalOAuth2Adapter)



from allauth.socialaccount.app_settings import STORE_TOKENS
from allauth.socialaccount.providers.base import ProviderAccount
from allauth.socialaccount.providers.eveonline.views import EveOnlineOAuth2Adapter
from allauth.socialaccount.providers.oauth2.provider import OAuth2Provider


class EveOnlineAccount(ProviderAccount):
    def get_profile_url(self):
        return "https://gate.eveonline.com/Profile/{char_name}".format(
            char_name=self.account.extra_data.get("CharacterName")
        )

    def get_avatar_url(self):
        return ("https://image.eveonline.com/Character/{char_id}_128.jpg").format(
            char_id=self.account.extra_data.get("CharacterID", 1)
        )

    def to_str(self):
        dflt = super().to_str()
        return next(
            value
            for value in (
                self.account.extra_data.get("CharacterName", None),
                self.account.extra_data.get("CharacterID", None),
                dflt,
            )
            if value is not None
        )


class EveOnlineProvider(OAuth2Provider):
    id = "eveonline"
    name = "EVE Online"
    account_class = EveOnlineAccount
    oauth2_adapter_class = EveOnlineOAuth2Adapter

    def get_default_scope(self):
        scopes = []
        if STORE_TOKENS:
            scopes.append("publicData")
        return scopes

    def extract_uid(self, data):
        return str(data["CharacterOwnerHash"])

    def extract_common_fields(self, data):
        return dict(name=data.get("CharacterName"))


provider_classes = [EveOnlineProvider]



from allauth.socialaccount.adapter import get_adapter
from allauth.socialaccount.providers.oauth2.views import (
    OAuth2Adapter,
    OAuth2CallbackView,
    OAuth2LoginView,
)

from .client import DingTalkOAuth2Client


class DingTalkOAuth2Adapter(OAuth2Adapter):
    provider_id = "dingtalk"
    access_token_url = "https://api.dingtalk.com/v1.0/oauth2/userAccessToken"  # nosec
    authorize_url = "https://login.dingtalk.com/oauth2/auth"
    profile_url = "https://api.dingtalk.com/v1.0/contact/users/me"
    client_class = DingTalkOAuth2Client

    def __init__(self, request):
        # dingtalk set "authCode" instead of "code" in callback url
        if "authCode" in request.GET:
            request.GET._mutable = True
            request.GET["code"] = request.GET["authCode"]
            request.GET._mutable = False

        super().__init__(request)

    def complete_login(self, request, app, token, **kwargs):
        headers = {"x-acs-dingtalk-access-token": token.token}
        with get_adapter().get_requests_session() as sess:
            resp = sess.get(self.profile_url, headers=headers)
            resp.raise_for_status()
            extra_data = resp.json()
        return self.get_provider().sociallogin_from_response(request, extra_data)


oauth2_login = OAuth2LoginView.adapter_view(DingTalkOAuth2Adapter)
oauth2_callback = OAuth2CallbackView.adapter_view(DingTalkOAuth2Adapter)



from allauth.socialaccount.adapter import get_adapter
from allauth.socialaccount.models import SocialToken
from allauth.socialaccount.providers.oauth2.views import (
    OAuth2Adapter,
    OAuth2CallbackView,
    OAuth2LoginView,
)


class AtlassianOAuth2Adapter(OAuth2Adapter):
    provider_id = "atlassian"
    access_token_url = "https://api.atlassian.com/oauth/token"  # nosec
    authorize_url = "https://auth.atlassian.com/authorize"
    profile_url = "https://api.atlassian.com/me"

    def complete_login(self, request, app, token: SocialToken, **kwargs):
        headers = {
            "Authorization": f"Bearer {token.token}",
            "Accept": "application/json",
        }
        with get_adapter().get_requests_session() as sess:
            response = sess.get(self.profile_url, headers=headers)
            response.raise_for_status()
            data = response.json()
        return self.get_provider().sociallogin_from_response(request, data)


oauth2_login = OAuth2LoginView.adapter_view(AtlassianOAuth2Adapter)
oauth2_callback = OAuth2CallbackView.adapter_view(AtlassianOAuth2Adapter)



from allauth.socialaccount.providers.oauth2.urls import default_urlpatterns
from allauth.socialaccount.providers.twentythreeandme.provider import (
    TwentyThreeAndMeProvider,
)


urlpatterns = default_urlpatterns(TwentyThreeAndMeProvider)



from allauth.socialaccount.sessions import LoginSession


APPLE_SESSION_COOKIE_NAME = "apple-login-session"


def get_apple_session(request):
    return LoginSession(request, "apple_login_session", APPLE_SESSION_COOKIE_NAME)



from allauth.socialaccount.providers.foursquare.provider import FoursquareProvider
from allauth.socialaccount.providers.oauth2.urls import default_urlpatterns


urlpatterns = default_urlpatterns(FoursquareProvider)



from allauth.socialaccount.providers.amazon.views import AmazonOAuth2Adapter
from allauth.socialaccount.providers.base import ProviderAccount
from allauth.socialaccount.providers.oauth2.provider import OAuth2Provider


class AmazonAccount(ProviderAccount):
    pass


class AmazonProvider(OAuth2Provider):
    id = "amazon"
    name = "Amazon"
    account_class = AmazonAccount
    oauth2_adapter_class = AmazonOAuth2Adapter

    def get_default_scope(self):
        return ["profile"]

    def extract_uid(self, data):
        return str(data["user_id"])

    def extract_common_fields(self, data):
        # Hackish way of splitting the fullname.
        # Assumes no middlenames.
        name = data.get("name", "")
        first_name, last_name = name, ""
        if name and " " in name:
            first_name, last_name = name.split(" ", 1)
        return dict(
            email=data.get("email", ""), last_name=last_name, first_name=first_name
        )


provider_classes = [AmazonProvider]



from allauth.socialaccount import app_settings
from allauth.socialaccount.adapter import get_adapter
from allauth.socialaccount.providers.oauth2.client import OAuth2Error
from allauth.socialaccount.providers.oauth2.views import (
    OAuth2Adapter,
    OAuth2CallbackView,
    OAuth2LoginView,
)
from allauth.socialaccount.providers.snapchat.constants import PROVIDER_ID, Scope


class SnapchatOAuth2Adapter(OAuth2Adapter):
    provider_id = PROVIDER_ID

    access_token_url = "https://accounts.snapchat.com/accounts/oauth2/token"  # nosec
    authorize_url = "https://accounts.snapchat.com/accounts/oauth2/auth"
    identity_url = "https://api.snapkit.com/v1/me"

    def complete_login(self, request, app, token, **kwargs):
        extra_data = self.get_data(token.token)
        return self.get_provider().sociallogin_from_response(request, extra_data)

    def get_data(self, token):
        settings = app_settings.PROVIDERS.get(self.provider_id, {})
        provider_scope = settings.get(
            "SCOPE",
            "['https://auth.snapchat.com/oauth2/api/user.external_id', 'https://auth.snapchat.com/oauth2/api/user.display_name']",
        )

        hed = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json;charset=UTF-8",
        }
        if Scope.BITMOJI in provider_scope:
            data = {"query": "{ me { externalId displayName bitmoji { avatar id } } }"}
        else:
            data = {"query": "{ me { externalId displayName } }"}

        with get_adapter().get_requests_session() as sess:
            resp = sess.post(self.identity_url, headers=hed, json=data)
            resp.raise_for_status()
            data = resp.json()

        if not data.get("data"):
            raise OAuth2Error()

        return data


oauth2_login = OAuth2LoginView.adapter_view(SnapchatOAuth2Adapter)
oauth2_callback = OAuth2CallbackView.adapter_view(SnapchatOAuth2Adapter)



from allauth.socialaccount import app_settings
from allauth.socialaccount.adapter import get_adapter
from allauth.socialaccount.providers.oauth2.views import (
    OAuth2Adapter,
    OAuth2CallbackView,
    OAuth2LoginView,
)


class TrainingPeaksOAuth2Adapter(OAuth2Adapter):
    # https://github.com/TrainingPeaks/PartnersAPI/wiki/OAuth
    provider_id = "trainingpeaks"

    def get_settings(self):
        """Provider settings"""
        return app_settings.PROVIDERS.get(self.provider_id, {})

    def get_hostname(self):
        """Return hostname depending on sandbox setting"""
        settings = self.get_settings()
        if settings.get("USE_PRODUCTION"):
            return "trainingpeaks.com"
        return "sandbox.trainingpeaks.com"

    @property
    def access_token_url(self):
        return f"https://oauth.{self.get_hostname()}/oauth/token"

    @property
    def authorize_url(self):
        return f"https://oauth.{self.get_hostname()}/OAuth/Authorize"

    @property
    def profile_url(self):
        return f"https://api.{self.get_hostname()}/v1/athlete/profile"

    @property
    def api_hostname(self):
        """Return https://api.hostname.tld"""
        return f"https://api.{self.get_hostname()}"

    # https://oauth.sandbox.trainingpeaks.com/oauth/deauthorize

    scope_delimiter = " "

    def complete_login(self, request, app, token, **kwargs):
        headers = {"Authorization": f"Bearer {token.token}"}
        with get_adapter().get_requests_session() as sess:
            response = sess.get(self.profile_url, headers=headers)
            response.raise_for_status()
            extra_data = response.json()
        return self.get_provider().sociallogin_from_response(request, extra_data)


oauth2_login = OAuth2LoginView.adapter_view(TrainingPeaksOAuth2Adapter)
oauth2_callback = OAuth2CallbackView.adapter_view(TrainingPeaksOAuth2Adapter)



import warnings

from allauth.core.ratelimit import clear, consume, consume_or_429


__all__ = ["consume", "consume_or_429", "clear"]
warnings.warn("allauth.ratelimit is deprecated, use allauth.core.ratelimit")



from threading import local

from django.contrib.auth import get_user_model
from django.contrib.auth.backends import ModelBackend

from allauth.account.adapter import get_adapter
from allauth.account.app_settings import LoginMethod

from . import app_settings
from .utils import filter_users_by_email, filter_users_by_username


_stash = local()


class AuthenticationBackend(ModelBackend):
    def authenticate(self, request, **credentials):
        password = credentials.get("password")
        if not password:
            return None
        self._did_check_password = False
        user = self._authenticate(request, **credentials)
        if not self._did_check_password:
            self._mitigate_timing_attack(password)
        return user

    def _authenticate(self, request, **credentials):
        password = credentials.get("password")
        username = credentials.get("username")
        if username:
            if LoginMethod.EMAIL in app_settings.LOGIN_METHODS:
                # Username/email ambiguity: even though allauth will pass along
                # `email` explicitly, other apps may not respect this. For example,
                # when using django-tastypie basic authentication, the login is
                # always passed as `username`.  So let's play nice with other apps
                # and use username as fallback.
                user = self._authenticate_by_email(username, password)
                if user:
                    return user
            user = self._authenticate_by_username(username, password)
            if user:
                return user

        email = credentials.get("email")
        if email:
            user = self._authenticate_by_email(email, password)
            if user:
                return user

        phone = credentials.get("phone")
        if phone:
            user = self._authenticate_by_phone(phone, password)
            if user:
                return user
        return None

    def _authenticate_by_phone(self, phone: str, password: str):
        if not phone or LoginMethod.PHONE not in app_settings.LOGIN_METHODS:
            return None
        adapter = get_adapter()
        user = adapter.get_user_by_phone(phone)
        return self._check_password(user, password)

    def _authenticate_by_username(self, username: str, password: str):
        if (
            (LoginMethod.USERNAME not in app_settings.LOGIN_METHODS)
            or (not app_settings.USER_MODEL_USERNAME_FIELD)
            or not username
        ):
            return None
        user = filter_users_by_username(username).first()
        return self._check_password(user, password)

    def _authenticate_by_email(
        self,
        email: str,
        password: str,
    ):
        if not email or LoginMethod.EMAIL not in app_settings.LOGIN_METHODS:
            return None
        users = filter_users_by_email(email, prefer_verified=True)
        for user in users:
            if self._check_password(user, password):
                return user
        return None

    def _mitigate_timing_attack(self, password):
        get_user_model()().set_password(password)

    def _check_password(self, user, password):
        if not user:
            return None
        self._did_check_password = True
        ok = user.check_password(password)
        if ok:
            ok = self.user_can_authenticate(user)
            if not ok:
                self._stash_user(user)
        return user if ok else None

    @classmethod
    def _stash_user(cls, user):
        """Now, be aware, the following is quite ugly, let me explain:

        Even if the user credentials match, the authentication can fail because
        Django's default ModelBackend calls user_can_authenticate(), which
        checks `is_active`. Now, earlier versions of allauth did not do this
        and simply returned the user as authenticated, even in case of
        `is_active=False`. For allauth scope, this does not pose a problem, as
        these users are properly redirected to an account inactive page.

        This does pose a problem when the allauth backend is used in a
        different context where allauth is not responsible for the login. Then,
        by not checking on `user_can_authenticate()` users will allow to become
        authenticated whereas according to Django logic this should not be
        allowed.

        In order to preserve the allauth behavior while respecting Django's
        logic, we stash a user for which the password check succeeded but
        `user_can_authenticate()` failed. In the allauth authentication logic,
        we can then unstash this user and proceed pointing the user to the
        account inactive page.
        """
        global _stash  # noqa: F824
        ret = getattr(_stash, "user", None)
        _stash.user = user
        return ret

    @classmethod
    def unstash_authenticated_user(cls):
        return cls._stash_user(None)



from allauth.account.models import EmailAddress
from allauth.socialaccount import app_settings
from allauth.socialaccount.providers.base import ProviderAccount
from allauth.socialaccount.providers.oauth2.provider import OAuth2Provider
from allauth.socialaccount.providers.yandex.views import YandexOAuth2Adapter


class YandexAccount(ProviderAccount):
    def to_str(self):
        email = self.account.extra_data.get("default_email")
        if email:
            return email
        return super().to_str()


class YandexProvider(OAuth2Provider):
    id = "yandex"
    name = "Yandex"
    account_class = YandexAccount
    oauth2_adapter_class = YandexOAuth2Adapter

    def get_default_scope(self):
        scope = ["login:info"]
        if app_settings.QUERY_EMAIL:
            scope.append("login:email")
        return scope

    def extract_uid(self, data):
        return str(data["id"])

    def get_user_email(self, data):
        email = data.get("default_email")
        if not email:
            emails = data.get("emails")
            email = emails[0] if emails else ""
        return email

    def extract_common_fields(self, data):
        email = self.get_user_email(data)
        return dict(
            email=email,
            last_name=data.get("last_name"),
            username=data.get("display_name"),
            first_name=data.get("first_name"),
        )

    def extract_email_addresses(self, data):
        ret = []
        email = self.get_user_email(data)
        if email:
            ret.append(EmailAddress(email=email, verified=True, primary=True))
        return ret


provider_classes = [YandexProvider]



from django.urls import include, path

from allauth.headless.socialaccount import views


def build_urlpatterns(client):
    return [
        path(
            "account/",
            include(
                [
                    path(
                        "providers",
                        views.ManageProvidersView.as_api_view(client=client),
                        name="manage_providers",
                    ),
                ]
            ),
        ),
        path(
            "auth/",
            include(
                [
                    path(
                        "provider/",
                        include(
                            [
                                path(
                                    "signup",
                                    views.ProviderSignupView.as_api_view(client=client),
                                    name="provider_signup",
                                ),
                                path(
                                    "redirect",
                                    views.RedirectToProviderView.as_api_view(
                                        client=client
                                    ),
                                    name="redirect_to_provider",
                                ),
                                path(
                                    "token",
                                    views.ProviderTokenView.as_api_view(client=client),
                                    name="provider_token",
                                ),
                            ]
                        ),
                    )
                ]
            ),
        ),
    ]



from django.urls import path

from . import views


urlpatterns = [
    path("telegram/login/", views.login, name="telegram_login"),
    path("telegram/login/callback/", views.callback, name="telegram_callback"),
]



import warnings

from allauth.headless.tokens.strategies.base import AbstractTokenStrategy


__all__ = ["AbstractTokenStrategy"]

warnings.warn(
    "allauth.headless.tokens.base.AbstractTokenStrategy is deprecated, use allauth.headless.tokens.strategies.base.AbstractTokenStrategy"
)



from allauth.socialaccount.providers.auth0.views import Auth0OAuth2Adapter
from allauth.socialaccount.providers.base import ProviderAccount
from allauth.socialaccount.providers.oauth2.provider import OAuth2Provider


class Auth0Account(ProviderAccount):
    def get_avatar_url(self):
        return self.account.extra_data.get("picture")


class Auth0Provider(OAuth2Provider):
    id = "auth0"
    name = "Auth0"
    account_class = Auth0Account
    oauth2_adapter_class = Auth0OAuth2Adapter

    def get_default_scope(self):
        return ["openid", "profile", "email"]

    def extract_uid(self, data):
        return str(data["sub"])

    def extract_common_fields(self, data):
        return dict(
            email=data.get("email"),
            username=data.get("username"),
            name=data.get("name"),
        )


provider_classes = [Auth0Provider]



from allauth.socialaccount.providers.oauth2.urls import default_urlpatterns
from allauth.socialaccount.providers.twitch.provider import TwitchProvider


urlpatterns = default_urlpatterns(TwitchProvider)



from allauth.socialaccount.providers.base import ProviderAccount
from allauth.socialaccount.providers.hubic.views import HubicOAuth2Adapter
from allauth.socialaccount.providers.oauth2.provider import OAuth2Provider


class HubicAccount(ProviderAccount):
    pass


class HubicProvider(OAuth2Provider):
    id = "hubic"
    name = "Hubic"
    account_class = HubicAccount
    oauth2_adapter_class = HubicOAuth2Adapter

    def extract_uid(self, data):
        return str(data["email"])

    def extract_common_fields(self, data):
        return dict(
            email=data.get("email"),
            username=data.get("firstname").lower() + data.get("lastname").lower(),
            first_name=data.get("firstname"),
            last_name=data.get("lastname"),
        )


provider_classes = [HubicProvider]



from allauth.socialaccount.providers.base import ProviderAccount
from allauth.socialaccount.providers.oauth2.provider import OAuth2Provider
from allauth.socialaccount.providers.twitch.views import TwitchOAuth2Adapter


class TwitchAccount(ProviderAccount):
    def get_profile_url(self):
        return f"https://twitch.tv/{self.account.extra_data.get('login')}"

    def get_avatar_url(self):
        # We're using `logo` as a failback for legacy profiles retrieved
        # with the old https://api.twitch.tv/kraken/user endpoint.
        logo = self.account.extra_data.get("logo")
        return self.account.extra_data.get("profile_image_url", logo)

    def to_str(self):
        dflt = super().to_str()
        return self.account.extra_data.get("login", dflt)


class TwitchProvider(OAuth2Provider):
    id = "twitch"
    name = "Twitch"
    account_class = TwitchAccount
    oauth2_adapter_class = TwitchOAuth2Adapter

    def extract_uid(self, data):
        return str(data["id"])

    def extract_common_fields(self, data):
        return {
            "username": data.get("login"),
            "name": data.get("display_name"),
            "email": data.get("email"),
        }

    def get_default_scope(self):
        return ["user:read:email"]


provider_classes = [TwitchProvider]



from allauth.socialaccount.providers.oauth.urls import default_urlpatterns
from allauth.socialaccount.providers.openstreetmap.provider import OpenStreetMapProvider


urlpatterns = default_urlpatterns(OpenStreetMapProvider)



import time

from django.contrib.auth import REDIRECT_FIELD_NAME
from django.http import HttpRequest, HttpResponseRedirect
from django.urls import reverse
from django.utils.http import urlencode

from allauth import app_settings as allauth_settings
from allauth.account import app_settings
from allauth.account.authentication import get_authentication_records
from allauth.account.internal.flows.login import record_authentication
from allauth.core.exceptions import ReauthenticationRequired
from allauth.core.internal.httpkit import deserialize_request, serialize_request
from allauth.core.internal.urlkit import script_aware_resolve
from allauth.utils import import_callable


STATE_SESSION_KEY = "account_reauthentication_state"


def reauthenticate_by_password(request: HttpRequest) -> None:
    record_authentication(
        request, request.user, method="password", reauthenticated=True
    )


def stash_and_reauthenticate(
    request: HttpRequest, state: dict, callback: str
) -> HttpResponseRedirect:
    request.session[STATE_SESSION_KEY] = {
        "state": state,
        "callback": callback,
    }
    return HttpResponseRedirect(reverse("account_reauthenticate"))


def suspend_request(request: HttpRequest, redirect_to: str) -> HttpResponseRedirect:
    path = request.get_full_path()
    if request.method == "POST":
        request.session[STATE_SESSION_KEY] = {"request": serialize_request(request)}
    return HttpResponseRedirect(
        f"{redirect_to}?{urlencode({REDIRECT_FIELD_NAME: path})}"
    )


def resume_request(request: HttpRequest) -> HttpResponseRedirect | None:
    from allauth.account.utils import get_next_redirect_url

    state = request.session.pop(STATE_SESSION_KEY, None)
    if state and "callback" in state:
        callback = import_callable(state["callback"])
        return callback(request, state["state"])

    url = get_next_redirect_url(request, REDIRECT_FIELD_NAME)
    if not url:
        return None
    if state and "request" in state:
        suspended_request = deserialize_request(state["request"], request)
        if suspended_request.path == url:
            resolved = script_aware_resolve(suspended_request.path)
            return resolved.func(suspended_request, *resolved.args, **resolved.kwargs)
    return HttpResponseRedirect(url)


def raise_if_reauthentication_required(request: HttpRequest) -> None:
    if not did_recently_authenticate(request):
        raise ReauthenticationRequired()


def did_recently_authenticate(request: HttpRequest) -> bool:
    if request.user.is_anonymous:
        return False
    if not get_reauthentication_flows(request.user):
        # TODO: This user only has social accounts attached. Now, ideally, you
        # would want to reauthenticate over at the social account provider. For
        # now, this is not implemented. Although definitely suboptimal, this
        # method is currently used for reauthentication checks over at MFA, and,
        # users that delegate the security of their account to an external
        # provider like Google typically use MFA over there anyway.
        return True
    methods = get_authentication_records(request)
    if not methods:
        return False
    authenticated_at = methods[-1]["at"]
    return time.time() - authenticated_at < app_settings.REAUTHENTICATION_TIMEOUT


def get_reauthentication_flows(user) -> list[dict]:
    ret: list[dict] = []
    if not user.is_authenticated:
        return ret
    if user.has_usable_password():
        entry = {
            "id": "reauthenticate",
        }
        ret.append(entry)
    if allauth_settings.MFA_ENABLED:
        from allauth.mfa.models import Authenticator
        from allauth.mfa.utils import is_mfa_enabled

        types = []
        for typ in Authenticator.Type:
            if is_mfa_enabled(user, types=[typ]):
                types.append(typ)
        if types:
            ret.append({"id": "mfa_reauthenticate", "types": types})
    return ret



from http import HTTPStatus

from allauth.headless.adapter import get_adapter
from allauth.headless.base.response import APIResponse


def email_address_data(addr) -> dict:
    return {
        "email": addr.email,
        "verified": addr.verified,
        "primary": addr.primary,
    }


class RequestEmailVerificationResponse(APIResponse):
    def __init__(self, request, verification_sent):
        super().__init__(
            request, status=HTTPStatus.OK if verification_sent else HTTPStatus.FORBIDDEN
        )


class VerifyEmailResponse(APIResponse):
    def __init__(self, request, email_address, stage):
        adapter = get_adapter()
        data = {
            "email": email_address.email,
            "user": adapter.serialize_user(email_address.user),
        }
        meta = {
            "is_authenticating": stage is not None,
        }
        super().__init__(request, data=data, meta=meta)


class EmailAddressesResponse(APIResponse):
    def __init__(self, request, email_addresses):
        data = [email_address_data(addr) for addr in email_addresses]
        super().__init__(request, data=data)


class PhoneNumbersResponse(APIResponse):
    def __init__(self, request, phone_numbers, status=HTTPStatus.OK):
        super().__init__(request, data=phone_numbers, status=status)


class RequestPasswordResponse(APIResponse):
    pass


class PasswordResetKeyResponse(APIResponse):
    def __init__(self, request, user):
        adapter = get_adapter()
        data = {"user": adapter.serialize_user(user)}
        super().__init__(request, data=data)



import requests

from django.urls import reverse
from django.utils.http import urlencode

from allauth.account.models import EmailAddress
from allauth.socialaccount.adapter import get_adapter
from allauth.socialaccount.internal import jwtkit
from allauth.socialaccount.providers.base import ProviderAccount
from allauth.socialaccount.providers.oauth2.client import OAuth2Error
from allauth.socialaccount.providers.oauth2.provider import OAuth2Provider
from allauth.socialaccount.providers.openid_connect.views import (
    OpenIDConnectOAuth2Adapter,
)


def _pick_data(data: dict) -> dict:
    # Prefer userinfo, it likely has more info compared to the ID token.
    if userinfo := data.get("userinfo"):
        return userinfo
    elif id_token := data.get("id_token"):
        return id_token
    # For backwards compatibility with <65.11
    return data


class OpenIDConnectProviderAccount(ProviderAccount):
    def get_user_data(self) -> dict | None:
        return _pick_data(self.account.extra_data)


class OpenIDConnectProvider(OAuth2Provider):
    id = "openid_connect"
    name = "OpenID Connect"
    account_class = OpenIDConnectProviderAccount
    oauth2_adapter_class = OpenIDConnectOAuth2Adapter
    supports_token_authentication = True

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.name = self.app.name

    @property
    def server_url(self):
        url = self.app.settings["server_url"]
        return self.wk_server_url(url)

    def wk_server_url(self, url):
        well_known_uri = "/.well-known/openid-configuration"
        if "/.well-known/" not in url:
            url += well_known_uri
        return url

    def get_login_url(self, request, **kwargs):
        url = reverse(
            f"{self.app.provider}_login", kwargs={"provider_id": self.app.provider_id}
        )
        if kwargs:
            url = f"{url}?{urlencode(kwargs)}"
        return url

    def get_callback_url(self):
        return reverse(
            f"{self.app.provider}_callback",
            kwargs={"provider_id": self.app.provider_id},
        )

    @property
    def token_auth_method(self):
        return self.app.settings.get("token_auth_method")

    def get_default_scope(self):
        return ["openid", "profile", "email"]

    def extract_uid(self, data):
        data = _pick_data(data)
        return str(data[self.app.settings.get("uid_field", "sub")])

    def extract_common_fields(self, data):
        data = _pick_data(data)
        return dict(
            email=data.get("email"),
            username=data.get("preferred_username"),
            name=data.get("name"),
            user_id=data.get("user_id"),
            picture=data.get("picture"),
            last_name=data.get("family_name"),
            first_name=data.get("given_name"),
        )

    def extract_email_addresses(self, data):
        data = _pick_data(data)
        addresses = []
        email = data.get("email")
        if email:
            addresses.append(
                EmailAddress(
                    email=email,
                    verified=data.get("email_verified", False),
                    primary=True,
                )
            )
        return addresses

    def get_oauth2_adapter(self, request):
        return self.oauth2_adapter_class(request, self.app.provider_id)

    def verify_token(self, request, token):
        id_token = token.get("id_token")
        if not id_token:
            raise get_adapter().validation_error("invalid_token")
        try:
            oauth2_adapter = self.get_oauth2_adapter(request)
            openid_config = oauth2_adapter.openid_config
            identity_data = jwtkit.verify_and_decode(
                credential=id_token,
                keys_url=openid_config["jwks_uri"],
                issuer=openid_config["issuer"],
                audience=[self.app.client_id],
                lookup_kid=jwtkit.lookup_kid_jwk,
            )
        except (OAuth2Error, requests.RequestException) as e:
            raise get_adapter().validation_error("invalid_token") from e
        login = self.sociallogin_from_response(request, identity_data)
        return login


provider_classes = [OpenIDConnectProvider]



from django.dispatch import Signal

from allauth.account import app_settings as account_settings
from allauth.mfa import app_settings
from allauth.mfa.adapter import get_adapter
from allauth.mfa.utils import is_mfa_enabled


# Emitted when an authenticator is added.
# Arguments: request, user, authenticator
authenticator_added = Signal()

# Emitted when an authenticator is removed.
# Arguments: request, user, authenticator
authenticator_removed = Signal()

# Emitted when an authenticator is reset (e.g. recovery codes regenerated).
# Arguments: request, user, authenticator
authenticator_reset = Signal()


def on_add_email(sender, email, user, **kwargs):
    if app_settings.ALLOW_UNVERIFIED_EMAIL:
        return
    if account_settings.EMAIL_VERIFICATION_BY_CODE_ENABLED:
        return
    if is_mfa_enabled(user):
        adapter = get_adapter()
        raise adapter.validation_error("add_email_blocked")



from allauth.socialaccount.providers.eveonline.provider import EveOnlineProvider
from allauth.socialaccount.providers.oauth2.urls import default_urlpatterns


urlpatterns = default_urlpatterns(EveOnlineProvider)



from allauth.account.models import EmailAddress
from allauth.socialaccount.providers.base import ProviderAccount
from allauth.socialaccount.providers.oauth2.provider import OAuth2Provider
from allauth.socialaccount.providers.okta.views import OktaOAuth2Adapter


class OktaAccount(ProviderAccount):
    pass


class OktaProvider(OAuth2Provider):
    id = "okta"
    name = "Okta"
    account_class = OktaAccount
    oauth2_adapter_class = OktaOAuth2Adapter

    def get_default_scope(self):
        return ["openid", "profile", "email", "offline_access"]

    def extract_uid(self, data):
        uid_field = self.app.settings.get("uid_field", "sub")
        return str(data[uid_field])

    def extract_extra_data(self, data):
        return data

    def extract_email_addresses(self, data):
        return [
            EmailAddress(
                email=data["email"], verified=bool(data["email_verified"]), primary=True
            )
        ]

    def extract_common_fields(self, data):
        ret = dict(
            email=data["email"],
            last_name=data["family_name"],
            first_name=data["given_name"],
        )
        preferred_username = data.get("preferred_username")
        if preferred_username:
            ret["username"] = preferred_username.partition("@")[0]
        return ret


provider_classes = [OktaProvider]



from allauth.socialaccount.providers.dwolla.provider import DwollaProvider
from allauth.socialaccount.providers.oauth2.urls import default_urlpatterns


urlpatterns = default_urlpatterns(DwollaProvider)



from allauth.socialaccount.adapter import get_adapter
from allauth.socialaccount.providers.oauth2.views import (
    OAuth2Adapter,
    OAuth2CallbackView,
    OAuth2LoginView,
)


class TwitterOAuth2Adapter(OAuth2Adapter):
    provider_id = "twitter_oauth2"
    access_token_url = "https://api.x.com/2/oauth2/token"  # nosec
    authorize_url = "https://x.com/i/oauth2/authorize"
    profile_url = "https://api.x.com/2/users/me"
    basic_auth = True

    def complete_login(self, request, app, access_token, **kwargs):
        extra_data = self.get_user_info(access_token)
        return self.get_provider().sociallogin_from_response(request, extra_data)

    def get_user_info(self, token):
        fields = self.get_provider().get_fields()
        headers = {
            **self.get_provider().get_settings().get("HEADERS", {}),
            "Authorization": f"Bearer {token.token}",
        }
        with get_adapter().get_requests_session() as sess:
            resp = sess.get(
                url=self.profile_url,
                params={"user.fields": ",".join(fields)},
                headers=headers,
            )
            resp.raise_for_status()
            return resp.json()["data"]


oauth2_login = OAuth2LoginView.adapter_view(TwitterOAuth2Adapter)
oauth2_callback = OAuth2CallbackView.adapter_view(TwitterOAuth2Adapter)



from allauth.socialaccount.providers.oauth2.urls import default_urlpatterns
from allauth.socialaccount.providers.yahoo.provider import YahooProvider


urlpatterns = default_urlpatterns(YahooProvider)



from allauth.socialaccount.providers.oauth.urls import default_urlpatterns
from allauth.socialaccount.providers.pocket.provider import PocketProvider


urlpatterns = default_urlpatterns(PocketProvider)



from allauth.socialaccount.providers.oauth2.urls import default_urlpatterns
from allauth.socialaccount.providers.pinterest.provider import PinterestProvider


urlpatterns = default_urlpatterns(PinterestProvider)



import warnings

from allauth.headless.tokens.strategies.sessions import SessionTokenStrategy


__all__ = ["SessionTokenStrategy"]

warnings.warn(
    "allauth.headless.tokens.sessions.SessionTokenStrategy is deprecated, use allauth.headless.tokens.strategies.sessions.SessionTokenStrategy"
)



from allauth.socialaccount.providers.amazon_cognito.provider import (
    AmazonCognitoProvider,
)
from allauth.socialaccount.providers.oauth2.urls import default_urlpatterns


urlpatterns = default_urlpatterns(AmazonCognitoProvider)



from http import HTTPStatus

from allauth.headless.base.response import APIResponse
from allauth.mfa import app_settings as mfa_settings


def get_config_data(request) -> dict:
    data = {
        "mfa": {
            "supported_types": mfa_settings.SUPPORTED_TYPES,
            "passkey_login_enabled": mfa_settings.PASSKEY_LOGIN_ENABLED,
        }
    }
    return data


def _authenticator_data(authenticator, sensitive: bool = False) -> dict:
    data = {
        "type": authenticator.type,
        "created_at": authenticator.created_at.timestamp(),
        "last_used_at": (
            authenticator.last_used_at.timestamp()
            if authenticator.last_used_at
            else None
        ),
    }
    if authenticator.type == authenticator.Type.TOTP:
        pass
    elif authenticator.type == authenticator.Type.RECOVERY_CODES:
        wrapped = authenticator.wrap()
        unused_codes = wrapped.get_unused_codes()
        data.update(
            {
                "total_code_count": len(wrapped.generate_codes()),
                "unused_code_count": len(unused_codes),
            }
        )
        if sensitive:
            data["unused_codes"] = unused_codes
    elif authenticator.type == authenticator.Type.WEBAUTHN:
        wrapped = authenticator.wrap()
        data["id"] = authenticator.pk
        data["name"] = wrapped.name
        passwordless = wrapped.is_passwordless
        if passwordless is not None:
            data["is_passwordless"] = passwordless
    return data


class AuthenticatorDeletedResponse(APIResponse):
    pass


class AuthenticatorsDeletedResponse(APIResponse):
    pass


class TOTPNotFoundResponse(APIResponse):
    def __init__(self, request, secret, totp_url):
        super().__init__(
            request,
            meta={
                "secret": secret,
                "totp_url": totp_url,
            },
            status=HTTPStatus.NOT_FOUND,
        )


class TOTPResponse(APIResponse):
    def __init__(self, request, authenticator):
        data = _authenticator_data(authenticator)
        super().__init__(request, data=data)


class AuthenticatorsResponse(APIResponse):
    def __init__(self, request, authenticators):
        data = [_authenticator_data(authenticator) for authenticator in authenticators]
        super().__init__(request, data=data)


class AuthenticatorResponse(APIResponse):
    def __init__(self, request, authenticator, meta=None):
        data = _authenticator_data(authenticator)
        super().__init__(request, data=data, meta=meta)


class RecoveryCodesNotFoundResponse(APIResponse):
    def __init__(self, request):
        super().__init__(request, status=HTTPStatus.NOT_FOUND)


class RecoveryCodesResponse(APIResponse):
    def __init__(self, request, authenticator):
        data = _authenticator_data(authenticator, sensitive=True)
        super().__init__(request, data=data)


class AddWebAuthnResponse(APIResponse):
    def __init__(self, request, registration_data):
        super().__init__(request, data={"creation_options": registration_data})


class WebAuthnRequestOptionsResponse(APIResponse):
    def __init__(self, request, request_options):
        super().__init__(request, data={"request_options": request_options})



from allauth.socialaccount.providers.coinbase.provider import CoinbaseProvider
from allauth.socialaccount.providers.oauth2.urls import default_urlpatterns


urlpatterns = default_urlpatterns(CoinbaseProvider)



from django.contrib import messages

from allauth.account import app_settings
from allauth.account.adapter import get_adapter
from allauth.account.internal.flows.code_verification import (
    AbstractCodeVerificationProcess,
)
from allauth.account.internal.flows.email_verification import verify_email_indirectly
from allauth.account.internal.flows.login import perform_login, record_authentication
from allauth.account.internal.flows.phone_verification import verify_phone_indirectly
from allauth.account.internal.flows.signup import send_unknown_account_mail
from allauth.account.internal.stagekit import clear_login, stash_login
from allauth.account.models import Login
from allauth.account.stages import LoginByCodeStage, LoginStageController


LOGIN_CODE_STATE_KEY = "login_code"


class LoginCodeVerificationProcess(AbstractCodeVerificationProcess):
    def __init__(self, stage):
        self.stage = stage
        self.request = stage.request
        super().__init__(
            state=stage.state,
            timeout=app_settings.LOGIN_BY_CODE_TIMEOUT,
            max_attempts=app_settings.LOGIN_BY_CODE_MAX_ATTEMPTS,
            user=stage.login.user,
        )

    def finish(self, redirect_url: str | None):
        email = self.state.get("email")
        phone = self.state.get("phone")
        user = self.user
        record_authentication(
            self.request, user, method="code", email=email, phone=phone
        )
        if email:
            verify_email_indirectly(self.request, user, email)
        if phone:
            verify_phone_indirectly(self.request, user, phone)
        if self.state["initiated_by_user"]:
            # Just requesting a login code does is not considered to be a real login,
            # yet, is needed in order to make the stage machinery work. Now that we've
            # completed the code, let's start a real login.
            login = Login(
                user=user,
                redirect_url=redirect_url,
                email=email,
            )
            return perform_login(self.request, login)
        else:
            return self.stage.exit()

    def abort(self) -> None:
        clear_login(self.request)

    def persist(self) -> None:
        stash_login(self.request, self.stage.login)

    def send(self, skip_enumeration_mails: bool = False) -> None:
        email = self.state.get("email")
        phone = self.state.get("phone")
        if email:
            self.send_by_email(email, skip_enumeration_mails=skip_enumeration_mails)
        elif phone:
            self.send_by_phone(phone, skip_enumeration_mails=skip_enumeration_mails)
        else:
            raise ValueError()

    def generate_code(self) -> str:
        email = self.state.get("email")
        phone = self.state.get("phone")
        adapter = get_adapter()
        if email:
            code = adapter.generate_login_code()
        elif phone:
            code = adapter._generate_phone_verification_code_compat(
                user=self.user, phone=phone
            )
        else:
            raise ValueError()
        self.state["code"] = code
        return code

    def send_by_phone(self, phone: str, skip_enumeration_mails: bool = False) -> None:
        adapter = get_adapter()
        if self.user:
            code = self.generate_code()
            adapter.send_verification_code_sms(user=self.user, phone=phone, code=code)
        elif not skip_enumeration_mails:
            if self.stage.login.signup:
                adapter.send_account_already_exists_sms(phone)
            else:
                adapter.send_unknown_account_sms(phone)
        self.add_sent_message({"recipient": phone, "phone": phone})

    def send_by_email(self, email: str, skip_enumeration_mails: bool = False) -> None:
        adapter = get_adapter()
        if self.user:
            code = self.generate_code()
            context = {
                "request": self.request,
                "code": code,
            }
            adapter.send_mail("account/email/login_code", email, context)
        elif not skip_enumeration_mails:
            if self.stage.login.signup:
                adapter.send_account_already_exists_mail(email)
            else:
                send_unknown_account_mail(self.request, email)
        self.add_sent_message({"email": email, "recipient": email})

    def add_sent_message(self, context) -> None:
        get_adapter().add_message(
            self.request,
            messages.SUCCESS,
            "account/messages/login_code_sent.txt",
            context,
        )

    @classmethod
    def initiate(
        cls,
        *,
        request,
        user,
        email: str | None = None,
        phone: str | None = None,
        stage=None,
    ):
        initial_state = cls.initial_state(user=user, email=email, phone=phone)
        initial_state["initiated_by_user"] = stage is None
        if not stage:
            login = Login(user=user, email=email)
            login.state["stages"] = {"current": "login_by_code"}
            stage = LoginByCodeStage(
                LoginStageController(request, login), request, login
            )
        stage.state.update(initial_state)
        process = LoginCodeVerificationProcess(stage=stage)
        process.send()
        process.persist()
        return process

    @classmethod
    def resume(cls, stage):
        process = LoginCodeVerificationProcess(stage=stage)
        return process.abort_if_invalid()

    @property
    def can_resend(self) -> bool:
        return not self.is_resend_quota_reached(
            app_settings.LOGIN_BY_CODE_MAX_RESEND_COUNT
        )

    def resend(self) -> None:
        self.generate_code()
        self.send(skip_enumeration_mails=True)
        self.record_resend()
        self.persist()



from allauth.socialaccount.adapter import get_adapter
from allauth.socialaccount.providers.oauth2.views import (
    OAuth2Adapter,
    OAuth2CallbackView,
    OAuth2LoginView,
)


class AmazonOAuth2Adapter(OAuth2Adapter):
    provider_id = "amazon"
    access_token_url = "https://api.amazon.com/auth/o2/token"  # nosec
    authorize_url = "https://www.amazon.com/ap/oa"
    profile_url = "https://api.amazon.com/user/profile"

    def complete_login(self, request, app, token, **kwargs):
        with get_adapter().get_requests_session() as sess:
            response = sess.get(self.profile_url, params={"access_token": token.token})
            response.raise_for_status()
            extra_data = response.json()
        if "Profile" in extra_data:
            extra_data = {
                "user_id": extra_data["Profile"]["CustomerId"],
                "name": extra_data["Profile"]["Name"],
                "email": extra_data["Profile"]["PrimaryEmail"],
            }
        return self.get_provider().sociallogin_from_response(request, extra_data)


oauth2_login = OAuth2LoginView.adapter_view(AmazonOAuth2Adapter)
oauth2_callback = OAuth2CallbackView.adapter_view(AmazonOAuth2Adapter)



from allauth.socialaccount.adapter import get_adapter
from allauth.socialaccount.providers.oauth2.views import (
    OAuth2Adapter,
    OAuth2CallbackView,
    OAuth2LoginView,
)


class CoinbaseOAuth2Adapter(OAuth2Adapter):
    provider_id = "coinbase"

    @property
    def authorize_url(self):
        return "https://www.coinbase.com/oauth/authorize"

    @property
    def access_token_url(self):
        return "https://www.coinbase.com/oauth/token"

    @property
    def profile_url(self):
        return "https://api.coinbase.com/v2/user"

    def complete_login(self, request, app, token, **kwargs):
        with get_adapter().get_requests_session() as sess:
            response = sess.get(self.profile_url, params={"access_token": token})
            extra_data = response.json()
        return self.get_provider().sociallogin_from_response(request, extra_data)


oauth2_login = OAuth2LoginView.adapter_view(CoinbaseOAuth2Adapter)
oauth2_callback = OAuth2CallbackView.adapter_view(CoinbaseOAuth2Adapter)



from django import forms
from django.contrib import admin

from allauth import app_settings
from allauth.account.adapter import get_adapter
from allauth.socialaccount import providers
from allauth.socialaccount.models import SocialAccount, SocialApp, SocialToken


class SocialAppForm(forms.ModelForm):
    class Meta:
        model = SocialApp
        exclude: list[str] = []
        widgets = {
            "client_id": forms.TextInput(attrs={"size": "100"}),
            "key": forms.TextInput(attrs={"size": "100"}),
            "secret": forms.TextInput(attrs={"size": "100"}),
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["provider"] = forms.ChoiceField(
            choices=providers.registry.as_choices()
        )


class SocialAppAdmin(admin.ModelAdmin):
    form = SocialAppForm
    list_display = (
        "name",
        "provider",
    )
    filter_horizontal = ("sites",) if app_settings.SITES_ENABLED else ()


class SocialAccountAdmin(admin.ModelAdmin):
    search_fields = ["uid"]
    raw_id_fields = ("user",)
    list_display = ("user", "uid", "provider")
    list_filter = ("provider",)

    def get_search_fields(self, request):
        search_fields = super().get_search_fields(request)
        user_search_fields = get_adapter().get_user_search_fields()
        return search_fields + list(map(lambda a: f"user__{a}", user_search_fields))


class SocialTokenAdmin(admin.ModelAdmin):
    raw_id_fields = (
        "app",
        "account",
    )
    list_display = ("app", "account", "truncated_token", "expires_at")
    list_filter = ("app", "app__provider", "expires_at")

    def truncated_token(self, token):
        max_chars = 40
        ret = token.token
        if len(ret) > max_chars:
            ret = f"{ret[0:max_chars]}...(truncated)"
        return ret

    truncated_token.short_description = "Token"  # type: ignore[attr-defined]


admin.site.register(SocialApp, SocialAppAdmin)
admin.site.register(SocialToken, SocialTokenAdmin)
admin.site.register(SocialAccount, SocialAccountAdmin)



from allauth.socialaccount.providers.base import ProviderAccount
from allauth.socialaccount.providers.daum.views import DaumOAuth2Adapter
from allauth.socialaccount.providers.oauth2.provider import OAuth2Provider


class DaumAccount(ProviderAccount):
    def get_avatar_url(self):
        return self.account.extra_data.get("bigImagePath")


class DaumProvider(OAuth2Provider):
    id = "Daum"
    name = "Daum"
    account_class = DaumAccount
    oauth2_adapter_class = DaumOAuth2Adapter

    def extract_uid(self, data):
        return str(data.get("id"))


provider_classes = [DaumProvider]



from allauth.socialaccount.providers.base import ProviderAccount
from allauth.socialaccount.providers.mailru.views import MailRuOAuth2Adapter
from allauth.socialaccount.providers.oauth2.provider import OAuth2Provider


class MailRuAccount(ProviderAccount):
    def get_profile_url(self):
        return self.account.extra_data.get("link")

    def get_avatar_url(self):
        ret = None
        if self.account.extra_data.get("has_pic"):
            pic_big_url = self.account.extra_data.get("pic_big")
            pic_small_url = self.account.extra_data.get("pic_small")
            if pic_big_url:
                return pic_big_url
            elif pic_small_url:
                return pic_small_url
        else:
            return ret


class MailRuProvider(OAuth2Provider):
    id = "mailru"
    name = "Mail.RU"
    account_class = MailRuAccount
    oauth2_adapter_class = MailRuOAuth2Adapter

    def extract_uid(self, data):
        return data["uid"]

    def extract_common_fields(self, data):
        return dict(
            email=data.get("email"),
            last_name=data.get("last_name"),
            username=data.get("nick"),
            first_name=data.get("first_name"),
        )


provider_classes = [MailRuProvider]



from allauth.headless.account.response import email_address_data
from allauth.headless.adapter import get_adapter
from allauth.headless.base.response import APIResponse
from allauth.headless.constants import Client, Flow
from allauth.socialaccount.adapter import get_adapter as get_socialaccount_adapter
from allauth.socialaccount.internal.flows import signup
from allauth.socialaccount.providers.oauth2.provider import OAuth2Provider


def _socialaccount_data(request, account):
    return {
        "uid": account.uid,
        "provider": _provider_data(request, account.get_provider()),
        "display": account.get_provider_account().to_str(),
    }


def _provider_data(request, provider):
    ret = {"id": provider.sub_id, "name": provider.name, "flows": []}
    if provider.supports_redirect:
        ret["flows"].append(Flow.PROVIDER_REDIRECT)
    if provider.supports_token_authentication:
        ret["flows"].append(Flow.PROVIDER_TOKEN)
    if isinstance(provider, OAuth2Provider):
        ret["client_id"] = provider.app.client_id
        if provider.id == "openid_connect":
            ret["openid_configuration_url"] = provider.server_url

    return ret


def provider_flows(request):
    flows = []
    providers = _list_supported_providers(request)
    if providers:
        redirect_providers = [p.sub_id for p in providers if p.supports_redirect]
        token_providers = [
            p.sub_id for p in providers if p.supports_token_authentication
        ]
        if redirect_providers and request.allauth.headless.client == Client.BROWSER:
            flows.append(
                {
                    "id": Flow.PROVIDER_REDIRECT,
                    "providers": redirect_providers,
                }
            )
        if token_providers:
            flows.append(
                {
                    "id": Flow.PROVIDER_TOKEN,
                    "providers": token_providers,
                }
            )
        sociallogin = signup.get_pending_signup(request)
        if sociallogin:
            flows.append(_signup_flow(request, sociallogin))
    return flows


def _signup_flow(request, sociallogin):
    provider = sociallogin.provider
    flow = {
        "id": Flow.PROVIDER_SIGNUP,
        "provider": _provider_data(request, provider),
        "is_pending": True,
    }
    return flow


def _is_provider_supported(provider, client):
    if client == Client.APP:
        return provider.supports_token_authentication
    elif client == Client.BROWSER:
        return provider.supports_redirect
    return False


def _list_supported_providers(request):
    adapter = get_socialaccount_adapter()
    providers = adapter.list_providers(request)
    providers = [
        p
        for p in providers
        if _is_provider_supported(p, request.allauth.headless.client)
    ]
    return providers


def get_config_data(request):
    entries = []
    data = {"socialaccount": {"providers": entries}}
    providers = _list_supported_providers(request)
    providers = sorted(providers, key=lambda p: p.name)
    for provider in providers:
        entries.append(_provider_data(request, provider))
    return data


class SocialAccountsResponse(APIResponse):
    def __init__(self, request, accounts):
        data = [_socialaccount_data(request, account) for account in accounts]
        super().__init__(request, data=data)


class SocialLoginResponse(APIResponse):
    def __init__(self, request, sociallogin):
        adapter = get_adapter()
        data = {
            "user": adapter.serialize_user(sociallogin.user),
            "account": _socialaccount_data(request, sociallogin.account),
            "email": [email_address_data(ea) for ea in sociallogin.email_addresses],
        }
        super().__init__(request, data=data)



from functools import wraps

from django.conf import settings
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.contrib.auth.decorators import login_required
from django.core.exceptions import PermissionDenied
from django.http import HttpResponseRedirect
from django.shortcuts import render, resolve_url
from django.urls import reverse

from allauth.account import app_settings
from allauth.account.internal.flows import reauthentication
from allauth.account.internal.flows.email_verification import (
    send_verification_email_for_user,
)
from allauth.account.models import EmailAddress
from allauth.account.utils import get_next_redirect_url
from allauth.core.exceptions import ReauthenticationRequired
from allauth.core.internal import httpkit


def verified_email_required(
    function=None, login_url=None, redirect_field_name=REDIRECT_FIELD_NAME
):
    """
    Even when email verification is not mandatory during signup, there
    may be circumstances during which you really want to prevent
    unverified users to proceed. This decorator ensures the user is
    authenticated and has a verified email address. If the former is
    not the case then the behavior is identical to that of the
    standard `login_required` decorator. If the latter does not hold,
    email verification mails are automatically resend and the user is
    presented with a page informing them they needs to verify their email
    address.
    """

    def decorator(view_func):
        @login_required(redirect_field_name=redirect_field_name, login_url=login_url)
        def _wrapped_view(request, *args, **kwargs):
            if not EmailAddress.objects.filter(
                user=request.user, verified=True
            ).exists():
                send_verification_email_for_user(request, request.user)
                if app_settings.EMAIL_VERIFICATION_BY_CODE_ENABLED:
                    url = httpkit.add_query_params(
                        reverse("account_email_verification_sent"),
                        {REDIRECT_FIELD_NAME: request.get_full_path()},
                    )
                    return HttpResponseRedirect(url)
                return render(request, "account/verified_email_required.html")
            return view_func(request, *args, **kwargs)

        return _wrapped_view

    if function:
        return decorator(function)
    return decorator


def reauthentication_required(
    function=None,
    redirect_field_name=REDIRECT_FIELD_NAME,
    allow_get=False,
    enabled=None,
):
    def decorator(view_func):
        @wraps(view_func)
        def _wrapper_view(request, *args, **kwargs):
            pass_method = allow_get and request.method == "GET"
            ena = (enabled is None) or (
                enabled(request) if callable(enabled) else enabled
            )
            if ena and not pass_method:
                if (
                    request.user.is_anonymous
                    or not reauthentication.did_recently_authenticate(request)
                ):
                    raise ReauthenticationRequired()
            return view_func(request, *args, **kwargs)

        return _wrapper_view

    if function:
        return decorator(function)
    return decorator


def secure_admin_login(function=None):
    def decorator(view_func):
        @wraps(view_func)
        def _wrapper_view(request, *args, **kwargs):
            if request.user.is_authenticated:
                if not request.user.is_staff or not request.user.is_active:
                    raise PermissionDenied()
                return view_func(request, *args, **kwargs)
            else:
                next_url = get_next_redirect_url(request)
                if not next_url:
                    next_url = request.get_full_path()
                login_url = resolve_url(settings.LOGIN_URL)
                login_url = httpkit.add_query_params(
                    login_url, {REDIRECT_FIELD_NAME: next_url}
                )
                return HttpResponseRedirect(login_url)

        return _wrapper_view

    if function:
        return decorator(function)
    return decorator



from django.urls import path

from allauth.headless import app_settings
from allauth.headless.spec import views


urlpatterns = [
    path("openapi.yaml", views.OpenAPIYAMLView.as_view(), name="openapi_yaml"),
    path("openapi.json", views.OpenAPIJSONView.as_view(), name="openapi_json"),
]

if app_settings.SERVE_SPECIFICATION and app_settings.SPECIFICATION_TEMPLATE_NAME:
    urlpatterns.append(
        path(
            "openapi.html",
            views.OpenAPIHTMLView.as_view(),
            name="openapi_html",
        ),
    )



from allauth.socialaccount.providers.flickr.provider import FlickrProvider
from allauth.socialaccount.providers.oauth.urls import default_urlpatterns


urlpatterns = default_urlpatterns(FlickrProvider)



from allauth.socialaccount.providers.basecamp.provider import BasecampProvider
from allauth.socialaccount.providers.oauth2.urls import default_urlpatterns


urlpatterns = default_urlpatterns(BasecampProvider)



from allauth.socialaccount.providers.oauth2.urls import default_urlpatterns
from allauth.socialaccount.providers.yandex.provider import YandexProvider


urlpatterns = default_urlpatterns(YandexProvider)



from allauth.socialaccount.providers.oauth2.urls import default_urlpatterns
from allauth.socialaccount.providers.orcid.provider import OrcidProvider


urlpatterns = default_urlpatterns(OrcidProvider)



from django.conf import settings
from django.db import migrations
from django.db.models.functions import Lower

from allauth.account import app_settings


def forwards(apps, schema_editor):
    EmailAddress = apps.get_model("account.EmailAddress")
    User = apps.get_model(settings.AUTH_USER_MODEL)
    EmailAddress.objects.all().exclude(email=Lower("email")).update(
        email=Lower("email")
    )
    email_field = app_settings.USER_MODEL_EMAIL_FIELD
    if email_field:
        User.objects.all().exclude(**{email_field: Lower(email_field)}).update(
            **{email_field: Lower(email_field)}
        )


class Migration(migrations.Migration):
    dependencies = [
        ("account", "0005_emailaddress_idx_upper_email"),
    ]

    operations = [migrations.RunPython(forwards, migrations.RunPython.noop)]



import requests

from allauth.account.models import EmailAddress
from allauth.socialaccount.adapter import get_adapter
from allauth.socialaccount.app_settings import QUERY_EMAIL
from allauth.socialaccount.providers.base import AuthAction, ProviderAccount
from allauth.socialaccount.providers.google.views import GoogleOAuth2Adapter
from allauth.socialaccount.providers.oauth2.client import OAuth2Error
from allauth.socialaccount.providers.oauth2.provider import OAuth2Provider


class Scope:
    EMAIL = "email"
    PROFILE = "profile"


class GoogleAccount(ProviderAccount):
    """
    The account data can be in two formats. One, originating from
    the /v2/userinfo endpoint:

        {'email': 'john.doe@gmail.com',
         'given_name': 'John',
         'id': '12345678901234567890',
         'locale': 'en',
         'name': 'John',
         'picture': 'https://lh3.googleusercontent.com/a/code',
         'verified_email': True}

    The second, which is the payload of the id_token:

        {'at_hash': '-someHASH',
         'aud': '123-pqr.apps.googleusercontent.com',
         'azp': '123-pqr.apps.googleusercontent.com',
         'email': 'john.doe@gmail.com',
         'email_verified': True,
         'exp': 1707297277,
         'given_name': 'John',
         'iat': 1707293677,
         'iss': 'https://accounts.google.com',
         'locale': 'en',
         'name': 'John',
         'picture': 'https://lh3.googleusercontent.com/a/code',
         'sub': '12345678901234567890'}
    """

    def get_profile_url(self):
        return self.account.extra_data.get("link")

    def get_avatar_url(self):
        return self.account.extra_data.get("picture")


class GoogleProvider(OAuth2Provider):
    id = "google"
    name = "Google"
    account_class = GoogleAccount
    oauth2_adapter_class = GoogleOAuth2Adapter
    supports_token_authentication = True

    def get_default_scope(self):
        scope = [Scope.PROFILE]
        if QUERY_EMAIL:
            scope.append(Scope.EMAIL)
        return scope

    def get_auth_params_from_request(self, request, action):
        ret = super().get_auth_params_from_request(request, action)
        if action == AuthAction.REAUTHENTICATE:
            ret["prompt"] = "select_account consent"
        return ret

    def extract_uid(self, data):
        if "sub" in data:
            return data["sub"]
        return data["id"]

    def extract_common_fields(self, data):
        return dict(
            email=data.get("email"),
            last_name=data.get("family_name"),
            first_name=data.get("given_name"),
        )

    def extract_email_addresses(self, data):
        ret = []
        email = data.get("email")
        if email:
            verified = bool(data.get("email_verified") or data.get("verified_email"))
            ret.append(EmailAddress(email=email, verified=verified, primary=True))
        return ret

    def verify_token(self, request, token):
        from allauth.socialaccount.providers.google import views

        credential = token.get("id_token")
        if not credential:
            raise get_adapter().validation_error("invalid_token")
        try:
            identity_data = views._verify_and_decode(
                app=self.app, credential=credential
            )
        except (OAuth2Error, requests.RequestException) as e:
            raise get_adapter().validation_error("invalid_token") from e
        login = self.sociallogin_from_response(request, identity_data)
        return login


provider_classes = [GoogleProvider]



from django.conf import settings
from django.urls import path, re_path

from allauth import app_settings as allauth_app_settings
from allauth.account import app_settings

from . import views


urlpatterns = [
    path("login/", views.login, name="account_login"),
    path("logout/", views.logout, name="account_logout"),
    path("inactive/", views.account_inactive, name="account_inactive"),
]

if not allauth_app_settings.SOCIALACCOUNT_ONLY:
    urlpatterns.extend(
        [
            path("signup/", views.signup, name="account_signup"),
            path(
                "reauthenticate/", views.reauthenticate, name="account_reauthenticate"
            ),
            # Email
            path("email/", views.email, name="account_email"),
            path(
                "confirm-email/",
                views.email_verification_sent,
                name="account_email_verification_sent",
            ),
            path(
                "password/change/",
                views.password_change,
                name="account_change_password",
            ),
            path("password/set/", views.password_set, name="account_set_password"),
            # password reset
            path(
                "password/reset/", views.password_reset, name="account_reset_password"
            ),
            path(
                "login/code/confirm/",
                views.confirm_login_code,
                name="account_confirm_login_code",
            ),
        ]
    )
    if not app_settings.EMAIL_VERIFICATION_BY_CODE_ENABLED:
        urlpatterns.append(
            re_path(
                r"^confirm-email/(?P<key>[-:\w]+)/$",
                views.confirm_email,
                name="account_confirm_email",
            )
        )
    if "phone" in app_settings.SIGNUP_FIELDS:
        urlpatterns.extend(
            [
                path(
                    "phone/verify/",
                    views.verify_phone,
                    name="account_verify_phone",
                ),
                path(
                    "phone/change/",
                    views.change_phone,
                    name="account_change_phone",
                ),
            ]
        )
    if app_settings.PASSWORD_RESET_BY_CODE_ENABLED:
        urlpatterns.extend(
            [
                path(
                    "password/reset/confirm/",
                    views.confirm_password_reset_code,
                    name="account_confirm_password_reset_code",
                ),
                path(
                    "password/reset/complete/",
                    views.complete_password_reset,
                    name="account_complete_password_reset",
                ),
                path(
                    "password/reset/done/",
                    views.password_reset_from_key_done,
                    name="account_password_reset_completed",
                ),
            ]
        )
    else:
        urlpatterns.extend(
            [
                path(
                    "password/reset/key/done/",
                    views.password_reset_from_key_done,
                    name="account_reset_password_from_key_done",
                ),
                re_path(
                    r"^password/reset/key/(?P<uidb36>[0-9A-Za-z]+)-(?P<key>.+)/$",
                    views.password_reset_from_key,
                    name="account_reset_password_from_key",
                ),
                path(
                    "password/reset/done/",
                    views.password_reset_done,
                    name="account_reset_password_done",
                ),
            ]
        )
    if getattr(settings, "MFA_PASSKEY_SIGNUP_ENABLED", False):
        urlpatterns.append(
            path(
                "signup/passkey/",
                views.signup_by_passkey,
                name="account_signup_by_passkey",
            )
        )

if app_settings.LOGIN_BY_CODE_ENABLED:
    urlpatterns.extend(
        [
            path(
                "login/code/",
                views.request_login_code,
                name="account_request_login_code",
            ),
        ]
    )



from django.contrib.auth import get_user
from django.contrib.sessions.backends.base import SessionBase
from django.http import HttpRequest


def get_session_user(session: SessionBase):
    request = HttpRequest()
    request.session = session
    user = get_user(request)
    if not user or user.is_anonymous:
        return None
    return user



from allauth.socialaccount import app_settings
from allauth.socialaccount.adapter import get_adapter
from allauth.socialaccount.providers.oauth2.views import (
    OAuth2Adapter,
    OAuth2CallbackView,
    OAuth2LoginView,
)


class FeedlyOAuth2Adapter(OAuth2Adapter):
    provider_id = "feedly"
    host = app_settings.PROVIDERS.get(provider_id, {}).get("HOST", "cloud.feedly.com")
    access_token_url = f"https://{host}/v3/auth/token"
    authorize_url = f"https://{host}/v3/auth/auth"
    profile_url = f"https://{host}/v3/profile"

    def complete_login(self, request, app, token, **kwargs):
        headers = {"Authorization": f"OAuth {token.token}"}
        with get_adapter().get_requests_session() as sess:
            resp = sess.get(self.profile_url, headers=headers)
            extra_data = resp.json()
        return self.get_provider().sociallogin_from_response(request, extra_data)


oauth2_login = OAuth2LoginView.adapter_view(FeedlyOAuth2Adapter)
oauth2_callback = OAuth2CallbackView.adapter_view(FeedlyOAuth2Adapter)



from django.conf import settings
from django.db import migrations
from django.db.models import Count


def forwards(apps, schema_editor):
    EmailAddress = apps.get_model("account.EmailAddress")
    User = apps.get_model(settings.AUTH_USER_MODEL)
    user_email_field = getattr(settings, "ACCOUNT_USER_MODEL_EMAIL_FIELD", "email")

    def get_users_with_multiple_primary_email():
        user_pks = []
        for email_address_dict in (
            EmailAddress.objects.filter(primary=True)
            .values("user")
            .annotate(Count("user"))
            .filter(user__count__gt=1)
        ):
            user_pks.append(email_address_dict["user"])
        return User.objects.filter(pk__in=user_pks)

    def unset_extra_primary_emails(user):
        qs = EmailAddress.objects.filter(user=user, primary=True)
        primary_email_addresses = list(qs)
        if not primary_email_addresses:
            return
        primary_email_address = primary_email_addresses[0]
        if user_email_field:
            for address in primary_email_addresses:
                if address.email.lower() == getattr(user, user_email_field, "").lower():
                    primary_email_address = address
                    break
        qs.exclude(pk=primary_email_address.pk).update(primary=False)

    for user in get_users_with_multiple_primary_email().iterator(2000):
        unset_extra_primary_emails(user)


class Migration(migrations.Migration):
    dependencies = [
        ("account", "0007_emailaddress_idx_email"),
    ]

    operations = [
        migrations.RunPython(code=forwards, reverse_code=migrations.RunPython.noop)
    ]



from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.http import HttpResponse, HttpResponseRedirect, JsonResponse
from django.urls import reverse, reverse_lazy
from django.utils.decorators import method_decorator
from django.views.generic.edit import DeleteView, FormView, UpdateView
from django.views.generic.list import ListView

from allauth.account import app_settings as account_settings
from allauth.account.adapter import get_adapter as get_account_adapter
from allauth.account.decorators import reauthentication_required
from allauth.account.internal.decorators import login_stage_required
from allauth.account.internal.templatekit import get_entrance_context_data
from allauth.account.mixins import NextRedirectMixin, RedirectAuthenticatedUserMixin
from allauth.account.models import Login
from allauth.account.views import BaseReauthenticateView
from allauth.mfa import app_settings
from allauth.mfa.internal.flows.add import redirect_if_add_not_allowed
from allauth.mfa.models import Authenticator
from allauth.mfa.webauthn.forms import (
    AddWebAuthnForm,
    EditWebAuthnForm,
    LoginWebAuthnForm,
    ReauthenticateWebAuthnForm,
    SignupWebAuthnForm,
)
from allauth.mfa.webauthn.internal import auth, flows
from allauth.mfa.webauthn.stages import PasskeySignupStage
from allauth.utils import get_form_class


@method_decorator(redirect_if_add_not_allowed, name="dispatch")
@method_decorator(reauthentication_required, name="dispatch")
class AddWebAuthnView(FormView):
    form_class = AddWebAuthnForm
    template_name = f"mfa/webauthn/add_form.{account_settings.TEMPLATE_EXTENSION}"

    def get_context_data(self, **kwargs) -> dict:
        ret = super().get_context_data()
        creation_options = auth.begin_registration(self.request.user, False)
        ret["js_data"] = {"creation_options": creation_options}
        return ret

    def get_form_class(self):
        return get_form_class(app_settings.FORMS, "add_webauthn", self.form_class)

    def get_form_kwargs(self) -> dict:
        ret = super().get_form_kwargs()
        ret["user"] = self.request.user
        return ret

    def get_success_url(self) -> str:
        if self.did_generate_recovery_codes:
            return reverse("mfa_view_recovery_codes")
        return reverse("mfa_index")

    def form_valid(self, form) -> HttpResponse:
        auth, rc_auth = flows.add_authenticator(
            self.request,
            name=form.cleaned_data["name"],
            credential=form.cleaned_data["credential"],
        )
        self.did_generate_recovery_codes = bool(rc_auth)
        return super().form_valid(form)


add_webauthn = AddWebAuthnView.as_view()


@method_decorator(login_required, name="dispatch")
class ListWebAuthnView(ListView):
    template_name = (
        f"mfa/webauthn/authenticator_list.{account_settings.TEMPLATE_EXTENSION}"
    )
    context_object_name = "authenticators"

    def get_queryset(self):
        return Authenticator.objects.filter(
            user=self.request.user, type=Authenticator.Type.WEBAUTHN
        )


list_webauthn = ListWebAuthnView.as_view()


@method_decorator(reauthentication_required, name="dispatch")
class RemoveWebAuthnView(NextRedirectMixin, DeleteView):
    object: Authenticator  # https://github.com/typeddjango/django-stubs/issues/1227
    template_name = (
        "mfa/webauthn/authenticator_confirm_delete."
        + account_settings.TEMPLATE_EXTENSION
    )
    success_url = reverse_lazy("mfa_list_webauthn")

    def get_queryset(self):
        return Authenticator.objects.filter(
            user=self.request.user, type=Authenticator.Type.WEBAUTHN
        )

    def form_valid(self, form) -> HttpResponse:
        authenticator = self.get_object()
        flows.remove_authenticator(self.request, authenticator)
        return HttpResponseRedirect(self.get_success_url())


remove_webauthn = RemoveWebAuthnView.as_view()


class LoginWebAuthnView(RedirectAuthenticatedUserMixin, FormView):
    form_class = LoginWebAuthnForm

    def get(self, request, *args, **kwargs) -> HttpResponse:
        if get_account_adapter().is_ajax(request):
            request_options = auth.begin_authentication(user=None)
            data = {"request_options": request_options}
            return JsonResponse(data)
        return HttpResponseRedirect(reverse("account_login"))

    def get_form_class(self):
        return get_form_class(app_settings.FORMS, "login_webauthn", self.form_class)

    def form_invalid(self, form) -> HttpResponse:
        for message in form.errors.get("credential", []):
            get_account_adapter().add_message(
                self.request, messages.ERROR, message=message
            )
        return HttpResponseRedirect(reverse("account_login"))

    def form_valid(self, form) -> HttpResponse:
        authenticator = form.cleaned_data["credential"]
        redirect_url = None
        login = Login(user=authenticator.user, redirect_url=redirect_url)
        return flows.perform_passwordless_login(self.request, authenticator, login)

    def get_context_data(self, **kwargs) -> dict:
        ret = super().get_context_data(**kwargs)
        ret.update(get_entrance_context_data(self.request))
        return ret


login_webauthn = LoginWebAuthnView.as_view()


@method_decorator(login_required, name="dispatch")
class ReauthenticateWebAuthnView(BaseReauthenticateView):
    form_class = ReauthenticateWebAuthnForm
    template_name = f"mfa/webauthn/reauthenticate.{account_settings.TEMPLATE_EXTENSION}"

    def get_form_class(self):
        return get_form_class(
            app_settings.FORMS, "reauthenticate_webauthn", self.form_class
        )

    def get_form_kwargs(self) -> dict:
        ret = super().get_form_kwargs()
        ret["user"] = self.request.user
        return ret

    def form_invalid(self, form) -> HttpResponse:
        for message in form.errors.get("credential", []):
            get_account_adapter().add_message(
                self.request, messages.ERROR, message=message
            )
        return HttpResponseRedirect(reverse("account_login"))

    def form_valid(self, form) -> HttpResponse:
        authenticator = form.cleaned_data["credential"]
        flows.reauthenticate(self.request, authenticator)
        return super().form_valid(form)

    def get_context_data(self, **kwargs) -> dict:
        ret = super().get_context_data()
        request_options = auth.begin_authentication(self.request.user)
        ret["js_data"] = {"request_options": request_options}
        return ret


reauthenticate_webauthn = ReauthenticateWebAuthnView.as_view()


@method_decorator(reauthentication_required, name="dispatch")
class EditWebAuthnView(NextRedirectMixin, UpdateView):
    form_class = EditWebAuthnForm
    template_name = f"mfa/webauthn/edit_form.{account_settings.TEMPLATE_EXTENSION}"
    success_url = reverse_lazy("mfa_list_webauthn")

    def get_form_class(self):
        return get_form_class(app_settings.FORMS, "edit_webauthn", self.form_class)

    def get_queryset(self):
        return Authenticator.objects.filter(
            user=self.request.user, type=Authenticator.Type.WEBAUTHN
        )


edit_webauthn = EditWebAuthnView.as_view()


@method_decorator(
    login_stage_required(
        stage=PasskeySignupStage.key, redirect_urlname="account_signup"
    ),
    name="dispatch",
)
class SignupWebAuthnView(FormView):
    form_class = SignupWebAuthnForm
    template_name = f"mfa/webauthn/signup_form.{account_settings.TEMPLATE_EXTENSION}"

    @property
    def _login_stage(self):
        return self.request._login_stage

    def get_context_data(self, **kwargs) -> dict:
        ret = super().get_context_data()
        ret.update(get_entrance_context_data(self.request))
        stage = self._login_stage
        creation_options = auth.begin_registration(stage.login.user, True)
        ret["js_data"] = {"creation_options": creation_options}
        return ret

    def get_form_class(self):
        return get_form_class(app_settings.FORMS, "signup_webauthn", self.form_class)

    def get_form_kwargs(self) -> dict:
        ret = super().get_form_kwargs()
        stage = self._login_stage
        ret["user"] = stage.login.user
        return ret

    def form_valid(self, form) -> HttpResponse:
        stage = self._login_stage
        flows.signup_authenticator(
            self.request,
            user=stage.login.user,
            name=form.cleaned_data["name"],
            credential=form.cleaned_data["credential"],
        )
        return stage.exit()


signup_webauthn = SignupWebAuthnView.as_view()



from django.contrib import messages
from django.contrib.auth import update_session_auth_hash
from django.contrib.auth.models import AbstractBaseUser
from django.http import HttpRequest

from allauth.account import app_settings, signals
from allauth.account.adapter import get_adapter
from allauth.account.internal.flows.logout import logout


def change_password(user: AbstractBaseUser, password: str) -> None:
    get_adapter().set_password(user, password)


def finalize_password_change(request: HttpRequest, user: AbstractBaseUser) -> bool:
    logged_out = logout_on_password_change(request, user)
    adapter = get_adapter(request)
    adapter.add_message(
        request,
        messages.SUCCESS,
        "account/messages/password_changed.txt",
    )
    adapter.send_notification_mail("account/email/password_changed", user)
    signals.password_changed.send(
        sender=user.__class__,
        request=request,
        user=user,
    )
    return logged_out


def finalize_password_set(request: HttpRequest, user: AbstractBaseUser) -> bool:
    logged_out = logout_on_password_change(request, user)
    adapter = get_adapter(request)
    adapter.add_message(request, messages.SUCCESS, "account/messages/password_set.txt")
    adapter.send_notification_mail("account/email/password_set", user)
    signals.password_set.send(
        sender=user.__class__,
        request=request,
        user=user,
    )
    return logged_out


def logout_on_password_change(request: HttpRequest, user: AbstractBaseUser) -> bool:
    # Since it is the default behavior of Django to invalidate all sessions on
    # password change, this function actually has to preserve the session when
    # logout isn't desired.
    logged_out = True
    if not app_settings.LOGOUT_ON_PASSWORD_CHANGE:
        update_session_auth_hash(request, user)  # type: ignore[arg-type]
        logged_out = False
    else:
        logout(request)
    return logged_out



from allauth.account.models import EmailAddress
from allauth.socialaccount import providers
from allauth.socialaccount.providers.base import AuthAction, ProviderAccount
from allauth.socialaccount.providers.oauth2.provider import OAuth2Provider
from allauth.socialaccount.providers.salesforce.views import SalesforceOAuth2Adapter


class SalesforceAccount(ProviderAccount):
    def get_profile_url(self):
        return self.account.extra_data.get("link")

    def get_avatar_url(self):
        return self.account.extra_data.get("picture")


class SalesforceProvider(OAuth2Provider):
    id = "salesforce"
    name = "Salesforce"
    package = "allauth.socialaccount.providers.salesforce"
    account_class = SalesforceAccount
    oauth2_adapter_class = SalesforceOAuth2Adapter

    def get_default_scope(self):
        return ["id", "openid"]

    def get_auth_params_from_request(self, request, action):
        ret = super().get_auth_params_from_request(request, action)
        if action == AuthAction.REAUTHENTICATE:
            ret["approval_prompt"] = "force"
        return ret

    def extract_uid(self, data):
        return str(data["user_id"])

    def extract_common_fields(self, data):
        return dict(
            email=data.get("email"),
            last_name=data.get("family_name"),
            first_name=data.get("given_name"),
            username=data.get("preferred_username"),
        )

    def extract_email_addresses(self, data):
        # a salesforce user must have an email, but it might not be verified
        email = EmailAddress(
            email=data.get("email"),
            primary=True,
            verified=data.get("email_verified"),
        )
        return [email]


providers.registry.register(SalesforceProvider)



from allauth.account.models import EmailAddress
from allauth.socialaccount.providers.base import ProviderAccount
from allauth.socialaccount.providers.hubspot.views import HubspotOAuth2Adapter
from allauth.socialaccount.providers.oauth2.provider import OAuth2Provider


class HubspotAccount(ProviderAccount):
    def to_str(self):
        return self.account.extra_data.get("user") or super().to_str()


class HubspotProvider(OAuth2Provider):
    id = "hubspot"
    name = "Hubspot"
    account_class = HubspotAccount
    oauth2_adapter_class = HubspotOAuth2Adapter

    def get_default_scope(self):
        return ["oauth"]

    def extract_uid(self, data):
        return str(data["user_id"])

    def extract_common_fields(self, data):
        return dict(email=data.get("user"))

    def extract_email_addresses(self, data):
        ret = []
        email = data.get("user")
        if email:
            ret.append(EmailAddress(email=email, verified=True, primary=True))
        return ret


provider_classes = [HubspotProvider]
